# Generated from /home/frances/projects/code/tarski/utils/parsers/grammars/fstrips.g4 by ANTLR 4.7.1
# encoding: utf-8
from antlr4 import *
from io import StringIO
from typing.io import TextIO
import sys

def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\u008c")
        buf.write("\u0336\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\4\64\t\64")
        buf.write("\4\65\t\65\4\66\t\66\4\67\t\67\48\t8\49\t9\4:\t:\4;\t")
        buf.write(";\4<\t<\4=\t=\4>\t>\4?\t?\4@\t@\4A\tA\4B\tB\4C\tC\3\2")
        buf.write("\3\2\5\2\u0089\n\2\3\3\3\3\3\3\3\3\5\3\u008f\n\3\3\3\5")
        buf.write("\3\u0092\n\3\3\3\5\3\u0095\n\3\3\3\5\3\u0098\n\3\3\3\5")
        buf.write("\3\u009b\n\3\3\3\7\3\u009e\n\3\f\3\16\3\u00a1\13\3\3\3")
        buf.write("\3\3\3\4\3\4\3\4\3\4\3\4\3\5\3\5\3\5\7\5\u00ad\n\5\f\5")
        buf.write("\16\5\u00b0\13\5\3\5\3\5\3\6\3\6\3\6\3\6\3\6\3\7\3\7\3")
        buf.write("\7\5\7\u00bc\n\7\3\b\3\b\5\b\u00c0\n\b\3\t\7\t\u00c3\n")
        buf.write("\t\f\t\16\t\u00c6\13\t\3\t\6\t\u00c9\n\t\r\t\16\t\u00ca")
        buf.write("\3\t\7\t\u00ce\n\t\f\t\16\t\u00d1\13\t\5\t\u00d3\n\t\3")
        buf.write("\n\6\n\u00d6\n\n\r\n\16\n\u00d7\3\n\3\n\3\n\3\13\7\13")
        buf.write("\u00de\n\13\f\13\16\13\u00e1\13\13\3\13\6\13\u00e4\n\13")
        buf.write("\r\13\16\13\u00e5\3\13\7\13\u00e9\n\13\f\13\16\13\u00ec")
        buf.write("\13\13\5\13\u00ee\n\13\3\f\7\f\u00f1\n\f\f\f\16\f\u00f4")
        buf.write("\13\f\3\f\6\f\u00f7\n\f\r\f\16\f\u00f8\3\f\7\f\u00fc\n")
        buf.write("\f\f\f\16\f\u00ff\13\f\5\f\u0101\n\f\3\r\6\r\u0104\n\r")
        buf.write("\r\r\16\r\u0105\3\r\3\r\3\r\3\16\3\16\3\16\6\16\u010e")
        buf.write("\n\16\r\16\16\16\u010f\3\16\3\16\3\16\5\16\u0115\n\16")
        buf.write("\3\17\3\17\5\17\u0119\n\17\3\20\3\20\3\20\7\20\u011e\n")
        buf.write("\20\f\20\16\20\u0121\13\20\3\20\3\20\3\21\3\21\5\21\u0127")
        buf.write("\n\21\3\22\3\22\3\22\3\22\3\22\3\22\3\22\3\23\3\23\3\23")
        buf.write("\3\23\3\23\3\24\3\24\3\25\3\25\3\25\3\25\3\25\3\26\3\26")
        buf.write("\3\26\7\26\u013f\n\26\f\26\16\26\u0142\13\26\3\26\3\26")
        buf.write("\3\27\3\27\3\27\3\27\3\27\3\30\3\30\3\31\3\31\3\32\3\32")
        buf.write("\3\32\3\32\5\32\u0153\n\32\3\33\3\33\3\33\3\33\3\33\3")
        buf.write("\33\3\33\3\33\3\33\3\33\3\34\3\34\3\34\3\34\3\34\3\34")
        buf.write("\3\34\3\34\3\34\3\34\3\34\3\35\3\35\3\35\3\35\3\35\3\35")
        buf.write("\3\35\3\35\3\35\3\35\3\36\3\36\3\37\3\37\3 \3 \3!\3!\3")
        buf.write("!\3!\3!\3\"\3\"\3\"\5\"\u0182\n\"\3#\3#\3#\3#\7#\u0188")
        buf.write("\n#\f#\16#\u018b\13#\3#\3#\3#\3#\7#\u0191\n#\f#\16#\u0194")
        buf.write("\13#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3")
        buf.write("#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\3#\5")
        buf.write("#\u01b8\n#\3$\3$\3$\7$\u01bd\n$\f$\16$\u01c0\13$\3$\3")
        buf.write("$\3%\3%\3%\3%\3%\5%\u01c9\n%\3&\3&\3&\7&\u01ce\n&\f&\16")
        buf.write("&\u01d1\13&\3&\3&\3&\3&\3&\3&\3&\3&\3&\3&\3&\3&\3&\5&")
        buf.write("\u01e0\n&\3\'\3\'\3\'\3\'\3\'\3\'\3\'\3\'\3\'\3(\3(\3")
        buf.write("(\7(\u01ee\n(\f(\16(\u01f1\13(\3(\3(\5(\u01f5\n(\3)\3")
        buf.write(")\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\3")
        buf.write(")\7)\u020b\n)\f)\16)\u020e\13)\3)\3)\3)\3)\5)\u0214\n")
        buf.write(")\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3")
        buf.write("*\5*\u0228\n*\3+\3+\3,\3,\3-\3-\3.\3.\3/\3/\3/\3/\3/\5")
        buf.write("/\u0237\n/\3/\5/\u023a\n/\3/\3/\3/\7/\u023f\n/\f/\16/")
        buf.write("\u0242\13/\3/\3/\3\60\3\60\3\60\5\60\u0249\n\60\3\61\3")
        buf.write("\61\3\61\3\61\3\61\3\62\3\62\3\62\3\62\3\62\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\64\3\64\3\64\6\64\u025d\n\64\r\64\16")
        buf.write("\64\u025e\3\64\3\64\3\65\3\65\3\65\3\65\3\65\3\65\3\65")
        buf.write("\3\65\3\65\3\65\3\65\3\66\3\66\3\66\7\66\u0271\n\66\f")
        buf.write("\66\16\66\u0274\13\66\3\66\3\66\3\67\3\67\3\67\3\67\3")
        buf.write("\67\3\67\3\67\3\67\3\67\3\67\3\67\3\67\5\67\u0284\n\67")
        buf.write("\38\38\38\78\u0289\n8\f8\168\u028c\138\38\38\39\39\39")
        buf.write("\79\u0293\n9\f9\169\u0296\139\39\39\3:\3:\5:\u029c\n:")
        buf.write("\3;\3;\3;\3;\3;\3<\3<\3<\3<\3<\3=\3=\3=\7=\u02ab\n=\f")
        buf.write("=\16=\u02ae\13=\3=\3=\3=\3=\3=\3=\3=\3=\3=\3=\3=\3=\5")
        buf.write("=\u02bc\n=\3=\3=\3=\3=\6=\u02c2\n=\r=\16=\u02c3\5=\u02c6")
        buf.write("\n=\3>\3>\3>\3>\3>\3>\3?\3?\3@\3@\3@\3@\3@\3@\3@\3@\3")
        buf.write("@\3@\3@\3@\5@\u02dc\n@\3A\3A\3A\3A\3A\3B\3B\3B\3B\3B\3")
        buf.write("C\3C\3C\6C\u02eb\nC\rC\16C\u02ec\3C\3C\3C\3C\3C\3C\3C")
        buf.write("\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3")
        buf.write("C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3")
        buf.write("C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3C\3")
        buf.write("C\3C\3C\3C\3C\3C\3C\3C\3C\5C\u0334\nC\3C\2\2D\2\4\6\b")
        buf.write("\n\f\16\20\22\24\26\30\32\34\36 \"$&(*,.\60\62\64\668")
        buf.write(":<>@BDFHJLNPRTVXZ\\^`bdfhjlnprtvxz|~\u0080\u0082\u0084")
        buf.write("\2\t\4\2SSUU\3\2UV\4\2\t\t\25\32\4\2\t\t\33#\3\2$(\3\2")
        buf.write("MP\3\2\64\65\2\u0353\2\u0088\3\2\2\2\4\u008a\3\2\2\2\6")
        buf.write("\u00a4\3\2\2\2\b\u00a9\3\2\2\2\n\u00b3\3\2\2\2\f\u00bb")
        buf.write("\3\2\2\2\16\u00bf\3\2\2\2\20\u00d2\3\2\2\2\22\u00d5\3")
        buf.write("\2\2\2\24\u00ed\3\2\2\2\26\u0100\3\2\2\2\30\u0103\3\2")
        buf.write("\2\2\32\u0114\3\2\2\2\34\u0118\3\2\2\2\36\u011a\3\2\2")
        buf.write("\2 \u0126\3\2\2\2\"\u0128\3\2\2\2$\u012f\3\2\2\2&\u0134")
        buf.write("\3\2\2\2(\u0136\3\2\2\2*\u013b\3\2\2\2,\u0145\3\2\2\2")
        buf.write(".\u014a\3\2\2\2\60\u014c\3\2\2\2\62\u0152\3\2\2\2\64\u0154")
        buf.write("\3\2\2\2\66\u015e\3\2\2\28\u0169\3\2\2\2:\u0173\3\2\2")
        buf.write("\2<\u0175\3\2\2\2>\u0177\3\2\2\2@\u0179\3\2\2\2B\u0181")
        buf.write("\3\2\2\2D\u01b7\3\2\2\2F\u01b9\3\2\2\2H\u01c8\3\2\2\2")
        buf.write("J\u01df\3\2\2\2L\u01e1\3\2\2\2N\u01f4\3\2\2\2P\u0213\3")
        buf.write("\2\2\2R\u0227\3\2\2\2T\u0229\3\2\2\2V\u022b\3\2\2\2X\u022d")
        buf.write("\3\2\2\2Z\u022f\3\2\2\2\\\u0231\3\2\2\2^\u0248\3\2\2\2")
        buf.write("`\u024a\3\2\2\2b\u024f\3\2\2\2d\u0254\3\2\2\2f\u0259\3")
        buf.write("\2\2\2h\u0262\3\2\2\2j\u026d\3\2\2\2l\u0283\3\2\2\2n\u0285")
        buf.write("\3\2\2\2p\u028f\3\2\2\2r\u029b\3\2\2\2t\u029d\3\2\2\2")
        buf.write("v\u02a2\3\2\2\2x\u02c5\3\2\2\2z\u02c7\3\2\2\2|\u02cd\3")
        buf.write("\2\2\2~\u02db\3\2\2\2\u0080\u02dd\3\2\2\2\u0082\u02e2")
        buf.write("\3\2\2\2\u0084\u0333\3\2\2\2\u0086\u0089\5\4\3\2\u0087")
        buf.write("\u0089\5\\/\2\u0088\u0086\3\2\2\2\u0088\u0087\3\2\2\2")
        buf.write("\u0089\3\3\2\2\2\u008a\u008b\7\3\2\2\u008b\u008c\7\4\2")
        buf.write("\2\u008c\u008e\5\6\4\2\u008d\u008f\5\b\5\2\u008e\u008d")
        buf.write("\3\2\2\2\u008e\u008f\3\2\2\2\u008f\u0091\3\2\2\2\u0090")
        buf.write("\u0092\5\n\6\2\u0091\u0090\3\2\2\2\u0091\u0092\3\2\2\2")
        buf.write("\u0092\u0094\3\2\2\2\u0093\u0095\5(\25\2\u0094\u0093\3")
        buf.write("\2\2\2\u0094\u0095\3\2\2\2\u0095\u0097\3\2\2\2\u0096\u0098")
        buf.write("\5*\26\2\u0097\u0096\3\2\2\2\u0097\u0098\3\2\2\2\u0098")
        buf.write("\u009a\3\2\2\2\u0099\u009b\5\36\20\2\u009a\u0099\3\2\2")
        buf.write("\2\u009a\u009b\3\2\2\2\u009b\u009f\3\2\2\2\u009c\u009e")
        buf.write("\5\62\32\2\u009d\u009c\3\2\2\2\u009e\u00a1\3\2\2\2\u009f")
        buf.write("\u009d\3\2\2\2\u009f\u00a0\3\2\2\2\u00a0\u00a2\3\2\2\2")
        buf.write("\u00a1\u009f\3\2\2\2\u00a2\u00a3\7\5\2\2\u00a3\5\3\2\2")
        buf.write("\2\u00a4\u00a5\7\3\2\2\u00a5\u00a6\7\6\2\2\u00a6\u00a7")
        buf.write("\7U\2\2\u00a7\u00a8\7\5\2\2\u00a8\7\3\2\2\2\u00a9\u00aa")
        buf.write("\7\3\2\2\u00aa\u00ae\7\7\2\2\u00ab\u00ad\7D\2\2\u00ac")
        buf.write("\u00ab\3\2\2\2\u00ad\u00b0\3\2\2\2\u00ae\u00ac\3\2\2\2")
        buf.write("\u00ae\u00af\3\2\2\2\u00af\u00b1\3\2\2\2\u00b0\u00ae\3")
        buf.write("\2\2\2\u00b1\u00b2\7\5\2\2\u00b2\t\3\2\2\2\u00b3\u00b4")
        buf.write("\7\3\2\2\u00b4\u00b5\7\b\2\2\u00b5\u00b6\5\24\13\2\u00b6")
        buf.write("\u00b7\7\5\2\2\u00b7\13\3\2\2\2\u00b8\u00bc\7Q\2\2\u00b9")
        buf.write("\u00bc\7R\2\2\u00ba\u00bc\7T\2\2\u00bb\u00b8\3\2\2\2\u00bb")
        buf.write("\u00b9\3\2\2\2\u00bb\u00ba\3\2\2\2\u00bc\r\3\2\2\2\u00bd")
        buf.write("\u00c0\5\f\7\2\u00be\u00c0\7S\2\2\u00bf\u00bd\3\2\2\2")
        buf.write("\u00bf\u00be\3\2\2\2\u00c0\17\3\2\2\2\u00c1\u00c3\7U\2")
        buf.write("\2\u00c2\u00c1\3\2\2\2\u00c3\u00c6\3\2\2\2\u00c4\u00c2")
        buf.write("\3\2\2\2\u00c4\u00c5\3\2\2\2\u00c5\u00d3\3\2\2\2\u00c6")
        buf.write("\u00c4\3\2\2\2\u00c7\u00c9\5\22\n\2\u00c8\u00c7\3\2\2")
        buf.write("\2\u00c9\u00ca\3\2\2\2\u00ca\u00c8\3\2\2\2\u00ca\u00cb")
        buf.write("\3\2\2\2\u00cb\u00cf\3\2\2\2\u00cc\u00ce\7U\2\2\u00cd")
        buf.write("\u00cc\3\2\2\2\u00ce\u00d1\3\2\2\2\u00cf\u00cd\3\2\2\2")
        buf.write("\u00cf\u00d0\3\2\2\2\u00d0\u00d3\3\2\2\2\u00d1\u00cf\3")
        buf.write("\2\2\2\u00d2\u00c4\3\2\2\2\u00d2\u00c8\3\2\2\2\u00d3\21")
        buf.write("\3\2\2\2\u00d4\u00d6\7U\2\2\u00d5\u00d4\3\2\2\2\u00d6")
        buf.write("\u00d7\3\2\2\2\u00d7\u00d5\3\2\2\2\u00d7\u00d8\3\2\2\2")
        buf.write("\u00d8\u00d9\3\2\2\2\u00d9\u00da\7\t\2\2\u00da\u00db\5")
        buf.write("\32\16\2\u00db\23\3\2\2\2\u00dc\u00de\t\2\2\2\u00dd\u00dc")
        buf.write("\3\2\2\2\u00de\u00e1\3\2\2\2\u00df\u00dd\3\2\2\2\u00df")
        buf.write("\u00e0\3\2\2\2\u00e0\u00ee\3\2\2\2\u00e1\u00df\3\2\2\2")
        buf.write("\u00e2\u00e4\5\22\n\2\u00e3\u00e2\3\2\2\2\u00e4\u00e5")
        buf.write("\3\2\2\2\u00e5\u00e3\3\2\2\2\u00e5\u00e6\3\2\2\2\u00e6")
        buf.write("\u00ea\3\2\2\2\u00e7\u00e9\t\2\2\2\u00e8\u00e7\3\2\2\2")
        buf.write("\u00e9\u00ec\3\2\2\2\u00ea\u00e8\3\2\2\2\u00ea\u00eb\3")
        buf.write("\2\2\2\u00eb\u00ee\3\2\2\2\u00ec\u00ea\3\2\2\2\u00ed\u00df")
        buf.write("\3\2\2\2\u00ed\u00e3\3\2\2\2\u00ee\25\3\2\2\2\u00ef\u00f1")
        buf.write("\7W\2\2\u00f0\u00ef\3\2\2\2\u00f1\u00f4\3\2\2\2\u00f2")
        buf.write("\u00f0\3\2\2\2\u00f2\u00f3\3\2\2\2\u00f3\u0101\3\2\2\2")
        buf.write("\u00f4\u00f2\3\2\2\2\u00f5\u00f7\5\30\r\2\u00f6\u00f5")
        buf.write("\3\2\2\2\u00f7\u00f8\3\2\2\2\u00f8\u00f6\3\2\2\2\u00f8")
        buf.write("\u00f9\3\2\2\2\u00f9\u00fd\3\2\2\2\u00fa\u00fc\7W\2\2")
        buf.write("\u00fb\u00fa\3\2\2\2\u00fc\u00ff\3\2\2\2\u00fd\u00fb\3")
        buf.write("\2\2\2\u00fd\u00fe\3\2\2\2\u00fe\u0101\3\2\2\2\u00ff\u00fd")
        buf.write("\3\2\2\2\u0100\u00f2\3\2\2\2\u0100\u00f6\3\2\2\2\u0101")
        buf.write("\27\3\2\2\2\u0102\u0104\7W\2\2\u0103\u0102\3\2\2\2\u0104")
        buf.write("\u0105\3\2\2\2\u0105\u0103\3\2\2\2\u0105\u0106\3\2\2\2")
        buf.write("\u0106\u0107\3\2\2\2\u0107\u0108\7\t\2\2\u0108\u0109\5")
        buf.write("\32\16\2\u0109\31\3\2\2\2\u010a\u010b\7\3\2\2\u010b\u010d")
        buf.write("\7\n\2\2\u010c\u010e\5\34\17\2\u010d\u010c\3\2\2\2\u010e")
        buf.write("\u010f\3\2\2\2\u010f\u010d\3\2\2\2\u010f\u0110\3\2\2\2")
        buf.write("\u0110\u0111\3\2\2\2\u0111\u0112\7\5\2\2\u0112\u0115\3")
        buf.write("\2\2\2\u0113\u0115\5\34\17\2\u0114\u010a\3\2\2\2\u0114")
        buf.write("\u0113\3\2\2\2\u0115\33\3\2\2\2\u0116\u0119\7U\2\2\u0117")
        buf.write("\u0119\5\16\b\2\u0118\u0116\3\2\2\2\u0118\u0117\3\2\2")
        buf.write("\2\u0119\35\3\2\2\2\u011a\u011b\7\3\2\2\u011b\u011f\7")
        buf.write("\13\2\2\u011c\u011e\5 \21\2\u011d\u011c\3\2\2\2\u011e")
        buf.write("\u0121\3\2\2\2\u011f\u011d\3\2\2\2\u011f\u0120\3\2\2\2")
        buf.write("\u0120\u0122\3\2\2\2\u0121\u011f\3\2\2\2\u0122\u0123\7")
        buf.write("\5\2\2\u0123\37\3\2\2\2\u0124\u0127\5\"\22\2\u0125\u0127")
        buf.write("\5$\23\2\u0126\u0124\3\2\2\2\u0126\u0125\3\2\2\2\u0127")
        buf.write("!\3\2\2\2\u0128\u0129\7\3\2\2\u0129\u012a\5&\24\2\u012a")
        buf.write("\u012b\5\26\f\2\u012b\u012c\7\5\2\2\u012c\u012d\7\t\2")
        buf.write("\2\u012d\u012e\5\34\17\2\u012e#\3\2\2\2\u012f\u0130\7")
        buf.write("\3\2\2\u0130\u0131\5&\24\2\u0131\u0132\5\26\f\2\u0132")
        buf.write("\u0133\7\5\2\2\u0133%\3\2\2\2\u0134\u0135\t\3\2\2\u0135")
        buf.write("\'\3\2\2\2\u0136\u0137\7\3\2\2\u0137\u0138\7\f\2\2\u0138")
        buf.write("\u0139\5\20\t\2\u0139\u013a\7\5\2\2\u013a)\3\2\2\2\u013b")
        buf.write("\u013c\7\3\2\2\u013c\u0140\7\r\2\2\u013d\u013f\5,\27\2")
        buf.write("\u013e\u013d\3\2\2\2\u013f\u0142\3\2\2\2\u0140\u013e\3")
        buf.write("\2\2\2\u0140\u0141\3\2\2\2\u0141\u0143\3\2\2\2\u0142\u0140")
        buf.write("\3\2\2\2\u0143\u0144\7\5\2\2\u0144+\3\2\2\2\u0145\u0146")
        buf.write("\7\3\2\2\u0146\u0147\5.\30\2\u0147\u0148\5\26\f\2\u0148")
        buf.write("\u0149\7\5\2\2\u0149-\3\2\2\2\u014a\u014b\7U\2\2\u014b")
        buf.write("/\3\2\2\2\u014c\u014d\7U\2\2\u014d\61\3\2\2\2\u014e\u0153")
        buf.write("\5\64\33\2\u014f\u0153\58\35\2\u0150\u0153\5L\'\2\u0151")
        buf.write("\u0153\5\66\34\2\u0152\u014e\3\2\2\2\u0152\u014f\3\2\2")
        buf.write("\2\u0152\u0150\3\2\2\2\u0152\u0151\3\2\2\2\u0153\63\3")
        buf.write("\2\2\2\u0154\u0155\7\3\2\2\u0155\u0156\7L\2\2\u0156\u0157")
        buf.write("\5:\36\2\u0157\u0158\7\16\2\2\u0158\u0159\7\3\2\2\u0159")
        buf.write("\u015a\5\26\f\2\u015a\u015b\7\5\2\2\u015b\u015c\5@!\2")
        buf.write("\u015c\u015d\7\5\2\2\u015d\65\3\2\2\2\u015e\u015f\7\3")
        buf.write("\2\2\u015f\u0160\7\17\2\2\u0160\u0161\5<\37\2\u0161\u0162")
        buf.write("\7\16\2\2\u0162\u0163\7\3\2\2\u0163\u0164\5\26\f\2\u0164")
        buf.write("\u0165\7\5\2\2\u0165\u0166\7\20\2\2\u0166\u0167\5D#\2")
        buf.write("\u0167\u0168\7\5\2\2\u0168\67\3\2\2\2\u0169\u016a\7\3")
        buf.write("\2\2\u016a\u016b\7\21\2\2\u016b\u016c\5> \2\u016c\u016d")
        buf.write("\7\16\2\2\u016d\u016e\7\3\2\2\u016e\u016f\5\26\f\2\u016f")
        buf.write("\u0170\7\5\2\2\u0170\u0171\5@!\2\u0171\u0172\7\5\2\2\u0172")
        buf.write("9\3\2\2\2\u0173\u0174\t\3\2\2\u0174;\3\2\2\2\u0175\u0176")
        buf.write("\7U\2\2\u0176=\3\2\2\2\u0177\u0178\t\3\2\2\u0178?\3\2")
        buf.write("\2\2\u0179\u017a\7\\\2\2\u017a\u017b\5B\"\2\u017b\u017c")
        buf.write("\7]\2\2\u017c\u017d\5N(\2\u017dA\3\2\2\2\u017e\u017f\7")
        buf.write("\3\2\2\u017f\u0182\7\5\2\2\u0180\u0182\5D#\2\u0181\u017e")
        buf.write("\3\2\2\2\u0181\u0180\3\2\2\2\u0182C\3\2\2\2\u0183\u01b8")
        buf.write("\5F$\2\u0184\u0185\7\3\2\2\u0185\u0189\7E\2\2\u0186\u0188")
        buf.write("\5D#\2\u0187\u0186\3\2\2\2\u0188\u018b\3\2\2\2\u0189\u0187")
        buf.write("\3\2\2\2\u0189\u018a\3\2\2\2\u018a\u018c\3\2\2\2\u018b")
        buf.write("\u0189\3\2\2\2\u018c\u01b8\7\5\2\2\u018d\u018e\7\3\2\2")
        buf.write("\u018e\u0192\7G\2\2\u018f\u0191\5D#\2\u0190\u018f\3\2")
        buf.write("\2\2\u0191\u0194\3\2\2\2\u0192\u0190\3\2\2\2\u0192\u0193")
        buf.write("\3\2\2\2\u0193\u0195\3\2\2\2\u0194\u0192\3\2\2\2\u0195")
        buf.write("\u01b8\7\5\2\2\u0196\u0197\7\3\2\2\u0197\u0198\7F\2\2")
        buf.write("\u0198\u0199\5D#\2\u0199\u019a\7\5\2\2\u019a\u01b8\3\2")
        buf.write("\2\2\u019b\u019c\7\3\2\2\u019c\u019d\7H\2\2\u019d\u019e")
        buf.write("\5D#\2\u019e\u019f\5D#\2\u019f\u01a0\7\5\2\2\u01a0\u01b8")
        buf.write("\3\2\2\2\u01a1\u01a2\7\3\2\2\u01a2\u01a3\7I\2\2\u01a3")
        buf.write("\u01a4\7\3\2\2\u01a4\u01a5\5\26\f\2\u01a5\u01a6\7\5\2")
        buf.write("\2\u01a6\u01a7\5D#\2\u01a7\u01a8\7\5\2\2\u01a8\u01b8\3")
        buf.write("\2\2\2\u01a9\u01aa\7\3\2\2\u01aa\u01ab\7J\2\2\u01ab\u01ac")
        buf.write("\7\3\2\2\u01ac\u01ad\5\26\f\2\u01ad\u01ae\7\5\2\2\u01ae")
        buf.write("\u01af\5D#\2\u01af\u01b0\7\5\2\2\u01b0\u01b8\3\2\2\2\u01b1")
        buf.write("\u01b2\7\3\2\2\u01b2\u01b3\5X-\2\u01b3\u01b4\5H%\2\u01b4")
        buf.write("\u01b5\5H%\2\u01b5\u01b6\7\5\2\2\u01b6\u01b8\3\2\2\2\u01b7")
        buf.write("\u0183\3\2\2\2\u01b7\u0184\3\2\2\2\u01b7\u018d\3\2\2\2")
        buf.write("\u01b7\u0196\3\2\2\2\u01b7\u019b\3\2\2\2\u01b7\u01a1\3")
        buf.write("\2\2\2\u01b7\u01a9\3\2\2\2\u01b7\u01b1\3\2\2\2\u01b8E")
        buf.write("\3\2\2\2\u01b9\u01ba\7\3\2\2\u01ba\u01be\5.\30\2\u01bb")
        buf.write("\u01bd\5H%\2\u01bc\u01bb\3\2\2\2\u01bd\u01c0\3\2\2\2\u01be")
        buf.write("\u01bc\3\2\2\2\u01be\u01bf\3\2\2\2\u01bf\u01c1\3\2\2\2")
        buf.write("\u01c0\u01be\3\2\2\2\u01c1\u01c2\7\5\2\2\u01c2G\3\2\2")
        buf.write("\2\u01c3\u01c9\7U\2\2\u01c4\u01c9\7X\2\2\u01c5\u01c9\7")
        buf.write("W\2\2\u01c6\u01c9\7\22\2\2\u01c7\u01c9\5J&\2\u01c8\u01c3")
        buf.write("\3\2\2\2\u01c8\u01c4\3\2\2\2\u01c8\u01c5\3\2\2\2\u01c8")
        buf.write("\u01c6\3\2\2\2\u01c8\u01c7\3\2\2\2\u01c9I\3\2\2\2\u01ca")
        buf.write("\u01cb\7\3\2\2\u01cb\u01cf\5&\24\2\u01cc\u01ce\5H%\2\u01cd")
        buf.write("\u01cc\3\2\2\2\u01ce\u01d1\3\2\2\2\u01cf\u01cd\3\2\2\2")
        buf.write("\u01cf\u01d0\3\2\2\2\u01d0\u01d2\3\2\2\2\u01d1\u01cf\3")
        buf.write("\2\2\2\u01d2\u01d3\7\5\2\2\u01d3\u01e0\3\2\2\2\u01d4\u01d5")
        buf.write("\7\3\2\2\u01d5\u01d6\5T+\2\u01d6\u01d7\5H%\2\u01d7\u01d8")
        buf.write("\5H%\2\u01d8\u01d9\7\5\2\2\u01d9\u01e0\3\2\2\2\u01da\u01db")
        buf.write("\7\3\2\2\u01db\u01dc\5V,\2\u01dc\u01dd\5H%\2\u01dd\u01de")
        buf.write("\7\5\2\2\u01de\u01e0\3\2\2\2\u01df\u01ca\3\2\2\2\u01df")
        buf.write("\u01d4\3\2\2\2\u01df\u01da\3\2\2\2\u01e0K\3\2\2\2\u01e1")
        buf.write("\u01e2\7\3\2\2\u01e2\u01e3\7\23\2\2\u01e3\u01e4\7\3\2")
        buf.write("\2\u01e4\u01e5\7U\2\2\u01e5\u01e6\5\26\f\2\u01e6\u01e7")
        buf.write("\7\5\2\2\u01e7\u01e8\5D#\2\u01e8\u01e9\7\5\2\2\u01e9M")
        buf.write("\3\2\2\2\u01ea\u01eb\7\3\2\2\u01eb\u01ef\7E\2\2\u01ec")
        buf.write("\u01ee\5P)\2\u01ed\u01ec\3\2\2\2\u01ee\u01f1\3\2\2\2\u01ef")
        buf.write("\u01ed\3\2\2\2\u01ef\u01f0\3\2\2\2\u01f0\u01f2\3\2\2\2")
        buf.write("\u01f1\u01ef\3\2\2\2\u01f2\u01f5\7\5\2\2\u01f3\u01f5\5")
        buf.write("P)\2\u01f4\u01ea\3\2\2\2\u01f4\u01f3\3\2\2\2\u01f5O\3")
        buf.write("\2\2\2\u01f6\u01f7\7\3\2\2\u01f7\u01f8\7J\2\2\u01f8\u01f9")
        buf.write("\7\3\2\2\u01f9\u01fa\5\26\f\2\u01fa\u01fb\7\5\2\2\u01fb")
        buf.write("\u01fc\5N(\2\u01fc\u01fd\7\5\2\2\u01fd\u0214\3\2\2\2\u01fe")
        buf.write("\u01ff\7\3\2\2\u01ff\u0200\7K\2\2\u0200\u0201\5D#\2\u0201")
        buf.write("\u0202\5R*\2\u0202\u0203\7\5\2\2\u0203\u0214\3\2\2\2\u0204")
        buf.write("\u0205\7\3\2\2\u0205\u0206\7K\2\2\u0206\u0207\5D#\2\u0207")
        buf.write("\u0208\7\3\2\2\u0208\u020c\7E\2\2\u0209\u020b\5R*\2\u020a")
        buf.write("\u0209\3\2\2\2\u020b\u020e\3\2\2\2\u020c\u020a\3\2\2\2")
        buf.write("\u020c\u020d\3\2\2\2\u020d\u020f\3\2\2\2\u020e\u020c\3")
        buf.write("\2\2\2\u020f\u0210\7\5\2\2\u0210\u0211\7\5\2\2\u0211\u0214")
        buf.write("\3\2\2\2\u0212\u0214\5R*\2\u0213\u01f6\3\2\2\2\u0213\u01fe")
        buf.write("\3\2\2\2\u0213\u0204\3\2\2\2\u0213\u0212\3\2\2\2\u0214")
        buf.write("Q\3\2\2\2\u0215\u0216\7\3\2\2\u0216\u0217\7\24\2\2\u0217")
        buf.write("\u0218\5J&\2\u0218\u0219\5H%\2\u0219\u021a\7\5\2\2\u021a")
        buf.write("\u0228\3\2\2\2\u021b\u021c\7\3\2\2\u021c\u021d\5Z.\2\u021d")
        buf.write("\u021e\5J&\2\u021e\u021f\5H%\2\u021f\u0220\7\5\2\2\u0220")
        buf.write("\u0228\3\2\2\2\u0221\u0222\7\3\2\2\u0222\u0223\7F\2\2")
        buf.write("\u0223\u0224\5F$\2\u0224\u0225\7\5\2\2\u0225\u0228\3\2")
        buf.write("\2\2\u0226\u0228\5F$\2\u0227\u0215\3\2\2\2\u0227\u021b")
        buf.write("\3\2\2\2\u0227\u0221\3\2\2\2\u0227\u0226\3\2\2\2\u0228")
        buf.write("S\3\2\2\2\u0229\u022a\t\4\2\2\u022aU\3\2\2\2\u022b\u022c")
        buf.write("\t\5\2\2\u022cW\3\2\2\2\u022d\u022e\t\6\2\2\u022eY\3\2")
        buf.write("\2\2\u022f\u0230\t\7\2\2\u0230[\3\2\2\2\u0231\u0232\7")
        buf.write("\3\2\2\u0232\u0233\7\4\2\2\u0233\u0234\5`\61\2\u0234\u0236")
        buf.write("\5b\62\2\u0235\u0237\5\b\5\2\u0236\u0235\3\2\2\2\u0236")
        buf.write("\u0237\3\2\2\2\u0237\u0239\3\2\2\2\u0238\u023a\5d\63\2")
        buf.write("\u0239\u0238\3\2\2\2\u0239\u023a\3\2\2\2\u023a\u023b\3")
        buf.write("\2\2\2\u023b\u023c\5j\66\2\u023c\u0240\5t;\2\u023d\u023f")
        buf.write("\5^\60\2\u023e\u023d\3\2\2\2\u023f\u0242\3\2\2\2\u0240")
        buf.write("\u023e\3\2\2\2\u0240\u0241\3\2\2\2\u0241\u0243\3\2\2\2")
        buf.write("\u0242\u0240\3\2\2\2\u0243\u0244\7\5\2\2\u0244]\3\2\2")
        buf.write("\2\u0245\u0249\5v<\2\u0246\u0249\5f\64\2\u0247\u0249\5")
        buf.write("z>\2\u0248\u0245\3\2\2\2\u0248\u0246\3\2\2\2\u0248\u0247")
        buf.write("\3\2\2\2\u0249_\3\2\2\2\u024a\u024b\7\3\2\2\u024b\u024c")
        buf.write("\7)\2\2\u024c\u024d\7U\2\2\u024d\u024e\7\5\2\2\u024ea")
        buf.write("\3\2\2\2\u024f\u0250\7\3\2\2\u0250\u0251\7*\2\2\u0251")
        buf.write("\u0252\7U\2\2\u0252\u0253\7\5\2\2\u0253c\3\2\2\2\u0254")
        buf.write("\u0255\7\3\2\2\u0255\u0256\7+\2\2\u0256\u0257\5\20\t\2")
        buf.write("\u0257\u0258\7\5\2\2\u0258e\3\2\2\2\u0259\u025a\7\3\2")
        buf.write("\2\u025a\u025c\7,\2\2\u025b\u025d\5h\65\2\u025c\u025b")
        buf.write("\3\2\2\2\u025d\u025e\3\2\2\2\u025e\u025c\3\2\2\2\u025e")
        buf.write("\u025f\3\2\2\2\u025f\u0260\3\2\2\2\u0260\u0261\7\5\2\2")
        buf.write("\u0261g\3\2\2\2\u0262\u0263\7\3\2\2\u0263\u0264\7U\2\2")
        buf.write("\u0264\u0265\7\t\2\2\u0265\u0266\5\f\7\2\u0266\u0267\7")
        buf.write("-\2\2\u0267\u0268\7X\2\2\u0268\u0269\7.\2\2\u0269\u026a")
        buf.write("\7X\2\2\u026a\u026b\7/\2\2\u026b\u026c\7\5\2\2\u026ci")
        buf.write("\3\2\2\2\u026d\u026e\7\3\2\2\u026e\u0272\7[\2\2\u026f")
        buf.write("\u0271\5l\67\2\u0270\u026f\3\2\2\2\u0271\u0274\3\2\2\2")
        buf.write("\u0272\u0270\3\2\2\2\u0272\u0273\3\2\2\2\u0273\u0275\3")
        buf.write("\2\2\2\u0274\u0272\3\2\2\2\u0275\u0276\7\5\2\2\u0276k")
        buf.write("\3\2\2\2\u0277\u0284\5p9\2\u0278\u0279\7\3\2\2\u0279\u027a")
        buf.write("\7F\2\2\u027a\u027b\5p9\2\u027b\u027c\7\5\2\2\u027c\u0284")
        buf.write("\3\2\2\2\u027d\u027e\7\3\2\2\u027e\u027f\7&\2\2\u027f")
        buf.write("\u0280\5n8\2\u0280\u0281\5r:\2\u0281\u0282\7\5\2\2\u0282")
        buf.write("\u0284\3\2\2\2\u0283\u0277\3\2\2\2\u0283\u0278\3\2\2\2")
        buf.write("\u0283\u027d\3\2\2\2\u0284m\3\2\2\2\u0285\u0286\7\3\2")
        buf.write("\2\u0286\u028a\5\60\31\2\u0287\u0289\5r:\2\u0288\u0287")
        buf.write("\3\2\2\2\u0289\u028c\3\2\2\2\u028a\u0288\3\2\2\2\u028a")
        buf.write("\u028b\3\2\2\2\u028b\u028d\3\2\2\2\u028c\u028a\3\2\2\2")
        buf.write("\u028d\u028e\7\5\2\2\u028eo\3\2\2\2\u028f\u0290\7\3\2")
        buf.write("\2\u0290\u0294\5.\30\2\u0291\u0293\5r:\2\u0292\u0291\3")
        buf.write("\2\2\2\u0293\u0296\3\2\2\2\u0294\u0292\3\2\2\2\u0294\u0295")
        buf.write("\3\2\2\2\u0295\u0297\3\2\2\2\u0296\u0294\3\2\2\2\u0297")
        buf.write("\u0298\7\5\2\2\u0298q\3\2\2\2\u0299\u029c\7U\2\2\u029a")
        buf.write("\u029c\7X\2\2\u029b\u0299\3\2\2\2\u029b\u029a\3\2\2\2")
        buf.write("\u029cs\3\2\2\2\u029d\u029e\7\3\2\2\u029e\u029f\7\60\2")
        buf.write("\2\u029f\u02a0\5D#\2\u02a0\u02a1\7\5\2\2\u02a1u\3\2\2")
        buf.write("\2\u02a2\u02a3\7\3\2\2\u02a3\u02a4\7\61\2\2\u02a4\u02a5")
        buf.write("\5x=\2\u02a5\u02a6\7\5\2\2\u02a6w\3\2\2\2\u02a7\u02a8")
        buf.write("\7\3\2\2\u02a8\u02ac\7E\2\2\u02a9\u02ab\5x=\2\u02aa\u02a9")
        buf.write("\3\2\2\2\u02ab\u02ae\3\2\2\2\u02ac\u02aa\3\2\2\2\u02ac")
        buf.write("\u02ad\3\2\2\2\u02ad\u02af\3\2\2\2\u02ae\u02ac\3\2\2\2")
        buf.write("\u02af\u02c6\7\5\2\2\u02b0\u02b1\7\3\2\2\u02b1\u02b2\7")
        buf.write("J\2\2\u02b2\u02b3\7\3\2\2\u02b3\u02b4\5\26\f\2\u02b4\u02b5")
        buf.write("\7\5\2\2\u02b5\u02b6\5x=\2\u02b6\u02b7\7\5\2\2\u02b7\u02c6")
        buf.write("\3\2\2\2\u02b8\u02b9\7\3\2\2\u02b9\u02bb\7\62\2\2\u02ba")
        buf.write("\u02bc\7U\2\2\u02bb\u02ba\3\2\2\2\u02bb\u02bc\3\2\2\2")
        buf.write("\u02bc\u02bd\3\2\2\2\u02bd\u02be\5\u0084C\2\u02be\u02bf")
        buf.write("\7\5\2\2\u02bf\u02c6\3\2\2\2\u02c0\u02c2\5\u0084C\2\u02c1")
        buf.write("\u02c0\3\2\2\2\u02c2\u02c3\3\2\2\2\u02c3\u02c1\3\2\2\2")
        buf.write("\u02c3\u02c4\3\2\2\2\u02c4\u02c6\3\2\2\2\u02c5\u02a7\3")
        buf.write("\2\2\2\u02c5\u02b0\3\2\2\2\u02c5\u02b8\3\2\2\2\u02c5\u02c1")
        buf.write("\3\2\2\2\u02c6y\3\2\2\2\u02c7\u02c8\7\3\2\2\u02c8\u02c9")
        buf.write("\7\63\2\2\u02c9\u02ca\5|?\2\u02ca\u02cb\5~@\2\u02cb\u02cc")
        buf.write("\7\5\2\2\u02cc{\3\2\2\2\u02cd\u02ce\t\b\2\2\u02ce}\3\2")
        buf.write("\2\2\u02cf\u02dc\5J&\2\u02d0\u02d1\5\u0080A\2\u02d1\u02d2")
        buf.write("\5\u0082B\2\u02d2\u02dc\3\2\2\2\u02d3\u02d4\5\u0082B\2")
        buf.write("\u02d4\u02d5\5\u0080A\2\u02d5\u02dc\3\2\2\2\u02d6\u02dc")
        buf.write("\7\66\2\2\u02d7\u02d8\7\3\2\2\u02d8\u02d9\7\67\2\2\u02d9")
        buf.write("\u02da\7U\2\2\u02da\u02dc\7\5\2\2\u02db\u02cf\3\2\2\2")
        buf.write("\u02db\u02d0\3\2\2\2\u02db\u02d3\3\2\2\2\u02db\u02d6\3")
        buf.write("\2\2\2\u02db\u02d7\3\2\2\2\u02dc\177\3\2\2\2\u02dd\u02de")
        buf.write("\7\3\2\2\u02de\u02df\78\2\2\u02df\u02e0\5J&\2\u02e0\u02e1")
        buf.write("\7\5\2\2\u02e1\u0081\3\2\2\2\u02e2\u02e3\7\3\2\2\u02e3")
        buf.write("\u02e4\79\2\2\u02e4\u02e5\5J&\2\u02e5\u02e6\7\5\2\2\u02e6")
        buf.write("\u0083\3\2\2\2\u02e7\u02e8\7\3\2\2\u02e8\u02ea\7E\2\2")
        buf.write("\u02e9\u02eb\5\u0084C\2\u02ea\u02e9\3\2\2\2\u02eb\u02ec")
        buf.write("\3\2\2\2\u02ec\u02ea\3\2\2\2\u02ec\u02ed\3\2\2\2\u02ed")
        buf.write("\u02ee\3\2\2\2\u02ee\u02ef\7\5\2\2\u02ef\u0334\3\2\2\2")
        buf.write("\u02f0\u02f1\7\3\2\2\u02f1\u02f2\7J\2\2\u02f2\u02f3\7")
        buf.write("\3\2\2\u02f3\u02f4\5\26\f\2\u02f4\u02f5\7\5\2\2\u02f5")
        buf.write("\u02f6\5\u0084C\2\u02f6\u02f7\7\5\2\2\u02f7\u0334\3\2")
        buf.write("\2\2\u02f8\u02f9\7\3\2\2\u02f9\u02fa\7:\2\2\u02fa\u02fb")
        buf.write("\5D#\2\u02fb\u02fc\7\5\2\2\u02fc\u0334\3\2\2\2\u02fd\u02fe")
        buf.write("\7\3\2\2\u02fe\u02ff\7;\2\2\u02ff\u0300\5D#\2\u0300\u0301")
        buf.write("\7\5\2\2\u0301\u0334\3\2\2\2\u0302\u0303\7\3\2\2\u0303")
        buf.write("\u0304\7<\2\2\u0304\u0305\5D#\2\u0305\u0306\7\5\2\2\u0306")
        buf.write("\u0334\3\2\2\2\u0307\u0308\7\3\2\2\u0308\u0309\7=\2\2")
        buf.write("\u0309\u030a\7X\2\2\u030a\u030b\5D#\2\u030b\u030c\7\5")
        buf.write("\2\2\u030c\u0334\3\2\2\2\u030d\u030e\7\3\2\2\u030e\u030f")
        buf.write("\7>\2\2\u030f\u0310\5D#\2\u0310\u0311\7\5\2\2\u0311\u0334")
        buf.write("\3\2\2\2\u0312\u0313\7\3\2\2\u0313\u0314\7?\2\2\u0314")
        buf.write("\u0315\5D#\2\u0315\u0316\5D#\2\u0316\u0317\7\5\2\2\u0317")
        buf.write("\u0334\3\2\2\2\u0318\u0319\7\3\2\2\u0319\u031a\7@\2\2")
        buf.write("\u031a\u031b\5D#\2\u031b\u031c\5D#\2\u031c\u031d\7\5\2")
        buf.write("\2\u031d\u0334\3\2\2\2\u031e\u031f\7\3\2\2\u031f\u0320")
        buf.write("\7A\2\2\u0320\u0321\7X\2\2\u0321\u0322\5D#\2\u0322\u0323")
        buf.write("\5D#\2\u0323\u0324\7\5\2\2\u0324\u0334\3\2\2\2\u0325\u0326")
        buf.write("\7\3\2\2\u0326\u0327\7B\2\2\u0327\u0328\7X\2\2\u0328\u0329")
        buf.write("\7X\2\2\u0329\u032a\5D#\2\u032a\u032b\7\5\2\2\u032b\u0334")
        buf.write("\3\2\2\2\u032c\u032d\7\3\2\2\u032d\u032e\7C\2\2\u032e")
        buf.write("\u032f\7X\2\2\u032f\u0330\5D#\2\u0330\u0331\7\5\2\2\u0331")
        buf.write("\u0334\3\2\2\2\u0332\u0334\5D#\2\u0333\u02e7\3\2\2\2\u0333")
        buf.write("\u02f0\3\2\2\2\u0333\u02f8\3\2\2\2\u0333\u02fd\3\2\2\2")
        buf.write("\u0333\u0302\3\2\2\2\u0333\u0307\3\2\2\2\u0333\u030d\3")
        buf.write("\2\2\2\u0333\u0312\3\2\2\2\u0333\u0318\3\2\2\2\u0333\u031e")
        buf.write("\3\2\2\2\u0333\u0325\3\2\2\2\u0333\u032c\3\2\2\2\u0333")
        buf.write("\u0332\3\2\2\2\u0334\u0085\3\2\2\2?\u0088\u008e\u0091")
        buf.write("\u0094\u0097\u009a\u009f\u00ae\u00bb\u00bf\u00c4\u00ca")
        buf.write("\u00cf\u00d2\u00d7\u00df\u00e5\u00ea\u00ed\u00f2\u00f8")
        buf.write("\u00fd\u0100\u0105\u010f\u0114\u0118\u011f\u0126\u0140")
        buf.write("\u0152\u0181\u0189\u0192\u01b7\u01be\u01c8\u01cf\u01df")
        buf.write("\u01ef\u01f4\u020c\u0213\u0227\u0236\u0239\u0240\u0248")
        buf.write("\u025e\u0272\u0283\u028a\u0294\u029b\u02ac\u02bb\u02c3")
        buf.write("\u02c5\u02db\u02ec\u0333")
        return buf.getvalue()


class fstripsParser ( Parser ):

    grammarFileName = "fstrips.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'('", "'define'", "')'", "'domain'", 
                     "':requirements'", "':types'", "'-'", "'either'", "':functions'", 
                     "':constants'", "':predicates'", "':parameters'", "':constraint'", 
                     "':condition'", "':event'", "'#t'", "':derived'", "'assign'", 
                     "'*'", "'+'", "'/'", "'^'", "'max'", "'min'", "'sin'", 
                     "'cos'", "'sqrt'", "'tan'", "'acos'", "'asin'", "'atan'", 
                     "'exp'", "'abs'", "'>'", "'<'", "'='", "'>='", "'<='", 
                     "'problem'", "':domain'", "':objects'", "':bounds'", 
                     "'['", "'..'", "']'", "':goal'", "':constraints'", 
                     "'preference'", "':metric'", "'minimize'", "'maximize'", 
                     "'(total-time)'", "'is-violated'", "':terminal'", "':stage'", 
                     "'at-end'", "'always'", "'sometime'", "'within'", "'at-most-once'", 
                     "'sometime-after'", "'sometime-before'", "'always-within'", 
                     "'hold-during'", "'hold-after'", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'scale-up'", "'scale-down'", "'int'", 
                     "'float'", "'object'", "'number'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "REQUIRE_KEY", "K_AND", 
                      "K_NOT", "K_OR", "K_IMPLY", "K_EXISTS", "K_FORALL", 
                      "K_WHEN", "K_ACTION", "K_INCREASE", "K_DECREASE", 
                      "K_SCALEUP", "K_SCALEDOWN", "INT_T", "FLOAT_T", "OBJECT_T", 
                      "NUMBER_T", "NAME", "EXTNAME", "VARIABLE", "NUMBER", 
                      "LINE_COMMENT", "WHITESPACE", "K_INIT", "K_PRECONDITION", 
                      "K_EFFECT", "DOMAIN", "DOMAIN_NAME", "REQUIREMENTS", 
                      "TYPES", "EITHER_TYPE", "CONSTANTS", "FUNCTIONS", 
                      "FREE_FUNCTIONS", "PREDICATES", "ACTION", "CONSTRAINT", 
                      "EVENT", "GLOBAL_CONSTRAINT", "DURATIVE_ACTION", "PROBLEM", 
                      "PROBLEM_NAME", "PROBLEM_DOMAIN", "OBJECTS", "INIT", 
                      "FUNC_HEAD", "PRECONDITION", "EFFECT", "AND_GD", "OR_GD", 
                      "NOT_GD", "IMPLY_GD", "EXISTS_GD", "FORALL_GD", "COMPARISON_GD", 
                      "AND_EFFECT", "FORALL_EFFECT", "WHEN_EFFECT", "ASSIGN_EFFECT", 
                      "NOT_EFFECT", "PRED_HEAD", "GOAL", "BINARY_OP", "EQUALITY_CON", 
                      "MULTI_OP", "MINUS_OP", "UNARY_MINUS", "INIT_EQ", 
                      "INIT_AT", "NOT_PRED_INIT", "PRED_INST", "PROBLEM_CONSTRAINT", 
                      "PROBLEM_METRIC" ]

    RULE_pddlDoc = 0
    RULE_domain = 1
    RULE_domainName = 2
    RULE_requireDef = 3
    RULE_declaration_of_types = 4
    RULE_numericBuiltinType = 5
    RULE_builtinType = 6
    RULE_possibly_typed_name_list = 7
    RULE_name_list_with_type = 8
    RULE_possibly_typed_type_list = 9
    RULE_possibly_typed_variable_list = 10
    RULE_variable_list_with_type = 11
    RULE_typename = 12
    RULE_primitive_type = 13
    RULE_function_definition_block = 14
    RULE_single_function_definition = 15
    RULE_typed_function_definition = 16
    RULE_untyped_function_definition = 17
    RULE_logical_symbol_name = 18
    RULE_constant_declaration = 19
    RULE_predicate_definition_block = 20
    RULE_single_predicate_definition = 21
    RULE_predicate = 22
    RULE_function_name = 23
    RULE_structureDef = 24
    RULE_actionDef = 25
    RULE_constraintDef = 26
    RULE_eventDef = 27
    RULE_actionName = 28
    RULE_constraintSymbol = 29
    RULE_eventSymbol = 30
    RULE_actionDefBody = 31
    RULE_precondition = 32
    RULE_goalDesc = 33
    RULE_atomicTermFormula = 34
    RULE_term = 35
    RULE_functionTerm = 36
    RULE_derivedDef = 37
    RULE_effect = 38
    RULE_single_effect = 39
    RULE_atomic_effect = 40
    RULE_builtin_binary_function = 41
    RULE_builtin_unary_function = 42
    RULE_builtin_binary_predicate = 43
    RULE_assignOp = 44
    RULE_problem = 45
    RULE_problemMeta = 46
    RULE_problemDecl = 47
    RULE_problemDomain = 48
    RULE_object_declaration = 49
    RULE_boundsDecl = 50
    RULE_typeBoundsDefinition = 51
    RULE_init = 52
    RULE_init_element = 53
    RULE_flat_term = 54
    RULE_flat_atom = 55
    RULE_constant_name = 56
    RULE_goal = 57
    RULE_probConstraints = 58
    RULE_prefConGD = 59
    RULE_metricSpec = 60
    RULE_optimization = 61
    RULE_metricFExp = 62
    RULE_terminalCost = 63
    RULE_stageCost = 64
    RULE_conGD = 65

    ruleNames =  [ "pddlDoc", "domain", "domainName", "requireDef", "declaration_of_types", 
                   "numericBuiltinType", "builtinType", "possibly_typed_name_list", 
                   "name_list_with_type", "possibly_typed_type_list", "possibly_typed_variable_list", 
                   "variable_list_with_type", "typename", "primitive_type", 
                   "function_definition_block", "single_function_definition", 
                   "typed_function_definition", "untyped_function_definition", 
                   "logical_symbol_name", "constant_declaration", "predicate_definition_block", 
                   "single_predicate_definition", "predicate", "function_name", 
                   "structureDef", "actionDef", "constraintDef", "eventDef", 
                   "actionName", "constraintSymbol", "eventSymbol", "actionDefBody", 
                   "precondition", "goalDesc", "atomicTermFormula", "term", 
                   "functionTerm", "derivedDef", "effect", "single_effect", 
                   "atomic_effect", "builtin_binary_function", "builtin_unary_function", 
                   "builtin_binary_predicate", "assignOp", "problem", "problemMeta", 
                   "problemDecl", "problemDomain", "object_declaration", 
                   "boundsDecl", "typeBoundsDefinition", "init", "init_element", 
                   "flat_term", "flat_atom", "constant_name", "goal", "probConstraints", 
                   "prefConGD", "metricSpec", "optimization", "metricFExp", 
                   "terminalCost", "stageCost", "conGD" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    T__21=22
    T__22=23
    T__23=24
    T__24=25
    T__25=26
    T__26=27
    T__27=28
    T__28=29
    T__29=30
    T__30=31
    T__31=32
    T__32=33
    T__33=34
    T__34=35
    T__35=36
    T__36=37
    T__37=38
    T__38=39
    T__39=40
    T__40=41
    T__41=42
    T__42=43
    T__43=44
    T__44=45
    T__45=46
    T__46=47
    T__47=48
    T__48=49
    T__49=50
    T__50=51
    T__51=52
    T__52=53
    T__53=54
    T__54=55
    T__55=56
    T__56=57
    T__57=58
    T__58=59
    T__59=60
    T__60=61
    T__61=62
    T__62=63
    T__63=64
    T__64=65
    REQUIRE_KEY=66
    K_AND=67
    K_NOT=68
    K_OR=69
    K_IMPLY=70
    K_EXISTS=71
    K_FORALL=72
    K_WHEN=73
    K_ACTION=74
    K_INCREASE=75
    K_DECREASE=76
    K_SCALEUP=77
    K_SCALEDOWN=78
    INT_T=79
    FLOAT_T=80
    OBJECT_T=81
    NUMBER_T=82
    NAME=83
    EXTNAME=84
    VARIABLE=85
    NUMBER=86
    LINE_COMMENT=87
    WHITESPACE=88
    K_INIT=89
    K_PRECONDITION=90
    K_EFFECT=91
    DOMAIN=92
    DOMAIN_NAME=93
    REQUIREMENTS=94
    TYPES=95
    EITHER_TYPE=96
    CONSTANTS=97
    FUNCTIONS=98
    FREE_FUNCTIONS=99
    PREDICATES=100
    ACTION=101
    CONSTRAINT=102
    EVENT=103
    GLOBAL_CONSTRAINT=104
    DURATIVE_ACTION=105
    PROBLEM=106
    PROBLEM_NAME=107
    PROBLEM_DOMAIN=108
    OBJECTS=109
    INIT=110
    FUNC_HEAD=111
    PRECONDITION=112
    EFFECT=113
    AND_GD=114
    OR_GD=115
    NOT_GD=116
    IMPLY_GD=117
    EXISTS_GD=118
    FORALL_GD=119
    COMPARISON_GD=120
    AND_EFFECT=121
    FORALL_EFFECT=122
    WHEN_EFFECT=123
    ASSIGN_EFFECT=124
    NOT_EFFECT=125
    PRED_HEAD=126
    GOAL=127
    BINARY_OP=128
    EQUALITY_CON=129
    MULTI_OP=130
    MINUS_OP=131
    UNARY_MINUS=132
    INIT_EQ=133
    INIT_AT=134
    NOT_PRED_INIT=135
    PRED_INST=136
    PROBLEM_CONSTRAINT=137
    PROBLEM_METRIC=138

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.7.1")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None



    class PddlDocContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def domain(self):
            return self.getTypedRuleContext(fstripsParser.DomainContext,0)


        def problem(self):
            return self.getTypedRuleContext(fstripsParser.ProblemContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_pddlDoc

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPddlDoc" ):
                listener.enterPddlDoc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPddlDoc" ):
                listener.exitPddlDoc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPddlDoc" ):
                return visitor.visitPddlDoc(self)
            else:
                return visitor.visitChildren(self)




    def pddlDoc(self):

        localctx = fstripsParser.PddlDocContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_pddlDoc)
        try:
            self.state = 134
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,0,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 132
                self.domain()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 133
                self.problem()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DomainContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def domainName(self):
            return self.getTypedRuleContext(fstripsParser.DomainNameContext,0)


        def requireDef(self):
            return self.getTypedRuleContext(fstripsParser.RequireDefContext,0)


        def declaration_of_types(self):
            return self.getTypedRuleContext(fstripsParser.Declaration_of_typesContext,0)


        def constant_declaration(self):
            return self.getTypedRuleContext(fstripsParser.Constant_declarationContext,0)


        def predicate_definition_block(self):
            return self.getTypedRuleContext(fstripsParser.Predicate_definition_blockContext,0)


        def function_definition_block(self):
            return self.getTypedRuleContext(fstripsParser.Function_definition_blockContext,0)


        def structureDef(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.StructureDefContext)
            else:
                return self.getTypedRuleContext(fstripsParser.StructureDefContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_domain

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDomain" ):
                listener.enterDomain(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDomain" ):
                listener.exitDomain(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDomain" ):
                return visitor.visitDomain(self)
            else:
                return visitor.visitChildren(self)




    def domain(self):

        localctx = fstripsParser.DomainContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_domain)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 136
            self.match(fstripsParser.T__0)
            self.state = 137
            self.match(fstripsParser.T__1)
            self.state = 138
            self.domainName()
            self.state = 140
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
            if la_ == 1:
                self.state = 139
                self.requireDef()


            self.state = 143
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,2,self._ctx)
            if la_ == 1:
                self.state = 142
                self.declaration_of_types()


            self.state = 146
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,3,self._ctx)
            if la_ == 1:
                self.state = 145
                self.constant_declaration()


            self.state = 149
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,4,self._ctx)
            if la_ == 1:
                self.state = 148
                self.predicate_definition_block()


            self.state = 152
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,5,self._ctx)
            if la_ == 1:
                self.state = 151
                self.function_definition_block()


            self.state = 157
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0:
                self.state = 154
                self.structureDef()
                self.state = 159
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 160
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DomainNameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_domainName

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDomainName" ):
                listener.enterDomainName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDomainName" ):
                listener.exitDomainName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDomainName" ):
                return visitor.visitDomainName(self)
            else:
                return visitor.visitChildren(self)




    def domainName(self):

        localctx = fstripsParser.DomainNameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_domainName)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 162
            self.match(fstripsParser.T__0)
            self.state = 163
            self.match(fstripsParser.T__3)
            self.state = 164
            self.match(fstripsParser.NAME)
            self.state = 165
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class RequireDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def REQUIRE_KEY(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.REQUIRE_KEY)
            else:
                return self.getToken(fstripsParser.REQUIRE_KEY, i)

        def getRuleIndex(self):
            return fstripsParser.RULE_requireDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRequireDef" ):
                listener.enterRequireDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRequireDef" ):
                listener.exitRequireDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitRequireDef" ):
                return visitor.visitRequireDef(self)
            else:
                return visitor.visitChildren(self)




    def requireDef(self):

        localctx = fstripsParser.RequireDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_requireDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 167
            self.match(fstripsParser.T__0)
            self.state = 168
            self.match(fstripsParser.T__4)
            self.state = 172
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.REQUIRE_KEY:
                self.state = 169
                self.match(fstripsParser.REQUIRE_KEY)
                self.state = 174
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 175
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Declaration_of_typesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def possibly_typed_type_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_type_listContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_declaration_of_types

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDeclaration_of_types" ):
                listener.enterDeclaration_of_types(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDeclaration_of_types" ):
                listener.exitDeclaration_of_types(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDeclaration_of_types" ):
                return visitor.visitDeclaration_of_types(self)
            else:
                return visitor.visitChildren(self)




    def declaration_of_types(self):

        localctx = fstripsParser.Declaration_of_typesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_declaration_of_types)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 177
            self.match(fstripsParser.T__0)
            self.state = 178
            self.match(fstripsParser.T__5)
            self.state = 179
            self.possibly_typed_type_list()
            self.state = 180
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class NumericBuiltinTypeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_numericBuiltinType

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class IntegerContext(NumericBuiltinTypeContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.NumericBuiltinTypeContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def INT_T(self):
            return self.getToken(fstripsParser.INT_T, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInteger" ):
                listener.enterInteger(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInteger" ):
                listener.exitInteger(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInteger" ):
                return visitor.visitInteger(self)
            else:
                return visitor.visitChildren(self)


    class FloatContext(NumericBuiltinTypeContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.NumericBuiltinTypeContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def FLOAT_T(self):
            return self.getToken(fstripsParser.FLOAT_T, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFloat" ):
                listener.enterFloat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFloat" ):
                listener.exitFloat(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFloat" ):
                return visitor.visitFloat(self)
            else:
                return visitor.visitChildren(self)


    class NumberContext(NumericBuiltinTypeContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.NumericBuiltinTypeContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER_T(self):
            return self.getToken(fstripsParser.NUMBER_T, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNumber" ):
                return visitor.visitNumber(self)
            else:
                return visitor.visitChildren(self)



    def numericBuiltinType(self):

        localctx = fstripsParser.NumericBuiltinTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_numericBuiltinType)
        try:
            self.state = 185
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.INT_T]:
                localctx = fstripsParser.IntegerContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 182
                self.match(fstripsParser.INT_T)
                pass
            elif token in [fstripsParser.FLOAT_T]:
                localctx = fstripsParser.FloatContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 183
                self.match(fstripsParser.FLOAT_T)
                pass
            elif token in [fstripsParser.NUMBER_T]:
                localctx = fstripsParser.NumberContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 184
                self.match(fstripsParser.NUMBER_T)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class BuiltinTypeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_builtinType

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class ObjectBuiltinContext(BuiltinTypeContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.BuiltinTypeContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def OBJECT_T(self):
            return self.getToken(fstripsParser.OBJECT_T, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterObjectBuiltin" ):
                listener.enterObjectBuiltin(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitObjectBuiltin" ):
                listener.exitObjectBuiltin(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitObjectBuiltin" ):
                return visitor.visitObjectBuiltin(self)
            else:
                return visitor.visitChildren(self)


    class NumericBuiltinContext(BuiltinTypeContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.BuiltinTypeContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def numericBuiltinType(self):
            return self.getTypedRuleContext(fstripsParser.NumericBuiltinTypeContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumericBuiltin" ):
                listener.enterNumericBuiltin(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumericBuiltin" ):
                listener.exitNumericBuiltin(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNumericBuiltin" ):
                return visitor.visitNumericBuiltin(self)
            else:
                return visitor.visitChildren(self)



    def builtinType(self):

        localctx = fstripsParser.BuiltinTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_builtinType)
        try:
            self.state = 189
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.INT_T, fstripsParser.FLOAT_T, fstripsParser.NUMBER_T]:
                localctx = fstripsParser.NumericBuiltinContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 187
                self.numericBuiltinType()
                pass
            elif token in [fstripsParser.OBJECT_T]:
                localctx = fstripsParser.ObjectBuiltinContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 188
                self.match(fstripsParser.OBJECT_T)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Possibly_typed_name_listContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_possibly_typed_name_list

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class SimpleNameListContext(Possibly_typed_name_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_name_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NAME)
            else:
                return self.getToken(fstripsParser.NAME, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSimpleNameList" ):
                listener.enterSimpleNameList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSimpleNameList" ):
                listener.exitSimpleNameList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSimpleNameList" ):
                return visitor.visitSimpleNameList(self)
            else:
                return visitor.visitChildren(self)


    class ComplexNameListContext(Possibly_typed_name_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_name_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def name_list_with_type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Name_list_with_typeContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Name_list_with_typeContext,i)

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NAME)
            else:
                return self.getToken(fstripsParser.NAME, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComplexNameList" ):
                listener.enterComplexNameList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComplexNameList" ):
                listener.exitComplexNameList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitComplexNameList" ):
                return visitor.visitComplexNameList(self)
            else:
                return visitor.visitChildren(self)



    def possibly_typed_name_list(self):

        localctx = fstripsParser.Possibly_typed_name_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_possibly_typed_name_list)
        self._la = 0 # Token type
        try:
            self.state = 208
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,13,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.SimpleNameListContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 194
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.NAME:
                    self.state = 191
                    self.match(fstripsParser.NAME)
                    self.state = 196
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                localctx = fstripsParser.ComplexNameListContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 198 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 197
                        self.name_list_with_type()

                    else:
                        raise NoViableAltException(self)
                    self.state = 200 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,11,self._ctx)

                self.state = 205
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.NAME:
                    self.state = 202
                    self.match(fstripsParser.NAME)
                    self.state = 207
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Name_list_with_typeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typename(self):
            return self.getTypedRuleContext(fstripsParser.TypenameContext,0)


        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NAME)
            else:
                return self.getToken(fstripsParser.NAME, i)

        def getRuleIndex(self):
            return fstripsParser.RULE_name_list_with_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterName_list_with_type" ):
                listener.enterName_list_with_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitName_list_with_type" ):
                listener.exitName_list_with_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitName_list_with_type" ):
                return visitor.visitName_list_with_type(self)
            else:
                return visitor.visitChildren(self)




    def name_list_with_type(self):

        localctx = fstripsParser.Name_list_with_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_name_list_with_type)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 211 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 210
                self.match(fstripsParser.NAME)
                self.state = 213 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==fstripsParser.NAME):
                    break

            self.state = 215
            self.match(fstripsParser.T__6)
            self.state = 216
            self.typename()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Possibly_typed_type_listContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_possibly_typed_type_list

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class UntypedTypenameListContext(Possibly_typed_type_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_type_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NAME)
            else:
                return self.getToken(fstripsParser.NAME, i)
        def OBJECT_T(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.OBJECT_T)
            else:
                return self.getToken(fstripsParser.OBJECT_T, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUntypedTypenameList" ):
                listener.enterUntypedTypenameList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUntypedTypenameList" ):
                listener.exitUntypedTypenameList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUntypedTypenameList" ):
                return visitor.visitUntypedTypenameList(self)
            else:
                return visitor.visitChildren(self)


    class TypedTypenameListContext(Possibly_typed_type_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_type_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def name_list_with_type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Name_list_with_typeContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Name_list_with_typeContext,i)

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NAME)
            else:
                return self.getToken(fstripsParser.NAME, i)
        def OBJECT_T(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.OBJECT_T)
            else:
                return self.getToken(fstripsParser.OBJECT_T, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTypedTypenameList" ):
                listener.enterTypedTypenameList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTypedTypenameList" ):
                listener.exitTypedTypenameList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTypedTypenameList" ):
                return visitor.visitTypedTypenameList(self)
            else:
                return visitor.visitChildren(self)



    def possibly_typed_type_list(self):

        localctx = fstripsParser.Possibly_typed_type_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_possibly_typed_type_list)
        self._la = 0 # Token type
        try:
            self.state = 235
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,18,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.UntypedTypenameListContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 221
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.OBJECT_T or _la==fstripsParser.NAME:
                    self.state = 218
                    _la = self._input.LA(1)
                    if not(_la==fstripsParser.OBJECT_T or _la==fstripsParser.NAME):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 223
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                localctx = fstripsParser.TypedTypenameListContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 225 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 224
                        self.name_list_with_type()

                    else:
                        raise NoViableAltException(self)
                    self.state = 227 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,16,self._ctx)

                self.state = 232
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.OBJECT_T or _la==fstripsParser.NAME:
                    self.state = 229
                    _la = self._input.LA(1)
                    if not(_la==fstripsParser.OBJECT_T or _la==fstripsParser.NAME):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 234
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Possibly_typed_variable_listContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_possibly_typed_variable_list

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class UntypedVariableListContext(Possibly_typed_variable_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_variable_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def VARIABLE(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.VARIABLE)
            else:
                return self.getToken(fstripsParser.VARIABLE, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUntypedVariableList" ):
                listener.enterUntypedVariableList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUntypedVariableList" ):
                listener.exitUntypedVariableList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUntypedVariableList" ):
                return visitor.visitUntypedVariableList(self)
            else:
                return visitor.visitChildren(self)


    class TypedVariableListContext(Possibly_typed_variable_listContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Possibly_typed_variable_listContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def variable_list_with_type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Variable_list_with_typeContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Variable_list_with_typeContext,i)

        def VARIABLE(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.VARIABLE)
            else:
                return self.getToken(fstripsParser.VARIABLE, i)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTypedVariableList" ):
                listener.enterTypedVariableList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTypedVariableList" ):
                listener.exitTypedVariableList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTypedVariableList" ):
                return visitor.visitTypedVariableList(self)
            else:
                return visitor.visitChildren(self)



    def possibly_typed_variable_list(self):

        localctx = fstripsParser.Possibly_typed_variable_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_possibly_typed_variable_list)
        self._la = 0 # Token type
        try:
            self.state = 254
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,22,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.UntypedVariableListContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 240
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.VARIABLE:
                    self.state = 237
                    self.match(fstripsParser.VARIABLE)
                    self.state = 242
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                localctx = fstripsParser.TypedVariableListContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 244 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 243
                        self.variable_list_with_type()

                    else:
                        raise NoViableAltException(self)
                    self.state = 246 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,20,self._ctx)

                self.state = 251
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.VARIABLE:
                    self.state = 248
                    self.match(fstripsParser.VARIABLE)
                    self.state = 253
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Variable_list_with_typeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typename(self):
            return self.getTypedRuleContext(fstripsParser.TypenameContext,0)


        def VARIABLE(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.VARIABLE)
            else:
                return self.getToken(fstripsParser.VARIABLE, i)

        def getRuleIndex(self):
            return fstripsParser.RULE_variable_list_with_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterVariable_list_with_type" ):
                listener.enterVariable_list_with_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitVariable_list_with_type" ):
                listener.exitVariable_list_with_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitVariable_list_with_type" ):
                return visitor.visitVariable_list_with_type(self)
            else:
                return visitor.visitChildren(self)




    def variable_list_with_type(self):

        localctx = fstripsParser.Variable_list_with_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_variable_list_with_type)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 257 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 256
                self.match(fstripsParser.VARIABLE)
                self.state = 259 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==fstripsParser.VARIABLE):
                    break

            self.state = 261
            self.match(fstripsParser.T__6)
            self.state = 262
            self.typename()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class TypenameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_typename

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class PrimitiveTypenameContext(TypenameContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TypenameContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def primitive_type(self):
            return self.getTypedRuleContext(fstripsParser.Primitive_typeContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPrimitiveTypename" ):
                listener.enterPrimitiveTypename(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPrimitiveTypename" ):
                listener.exitPrimitiveTypename(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPrimitiveTypename" ):
                return visitor.visitPrimitiveTypename(self)
            else:
                return visitor.visitChildren(self)


    class EitherTypenameContext(TypenameContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TypenameContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def primitive_type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Primitive_typeContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Primitive_typeContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEitherTypename" ):
                listener.enterEitherTypename(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEitherTypename" ):
                listener.exitEitherTypename(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEitherTypename" ):
                return visitor.visitEitherTypename(self)
            else:
                return visitor.visitChildren(self)



    def typename(self):

        localctx = fstripsParser.TypenameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_typename)
        self._la = 0 # Token type
        try:
            self.state = 274
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.T__0]:
                localctx = fstripsParser.EitherTypenameContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 264
                self.match(fstripsParser.T__0)
                self.state = 265
                self.match(fstripsParser.T__7)
                self.state = 267 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 266
                    self.primitive_type()
                    self.state = 269 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (((((_la - 79)) & ~0x3f) == 0 and ((1 << (_la - 79)) & ((1 << (fstripsParser.INT_T - 79)) | (1 << (fstripsParser.FLOAT_T - 79)) | (1 << (fstripsParser.OBJECT_T - 79)) | (1 << (fstripsParser.NUMBER_T - 79)) | (1 << (fstripsParser.NAME - 79)))) != 0)):
                        break

                self.state = 271
                self.match(fstripsParser.T__2)
                pass
            elif token in [fstripsParser.INT_T, fstripsParser.FLOAT_T, fstripsParser.OBJECT_T, fstripsParser.NUMBER_T, fstripsParser.NAME]:
                localctx = fstripsParser.PrimitiveTypenameContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 273
                self.primitive_type()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Primitive_typeContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def builtinType(self):
            return self.getTypedRuleContext(fstripsParser.BuiltinTypeContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_primitive_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPrimitive_type" ):
                listener.enterPrimitive_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPrimitive_type" ):
                listener.exitPrimitive_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPrimitive_type" ):
                return visitor.visitPrimitive_type(self)
            else:
                return visitor.visitChildren(self)




    def primitive_type(self):

        localctx = fstripsParser.Primitive_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_primitive_type)
        try:
            self.state = 278
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.NAME]:
                self.enterOuterAlt(localctx, 1)
                self.state = 276
                self.match(fstripsParser.NAME)
                pass
            elif token in [fstripsParser.INT_T, fstripsParser.FLOAT_T, fstripsParser.OBJECT_T, fstripsParser.NUMBER_T]:
                self.enterOuterAlt(localctx, 2)
                self.state = 277
                self.builtinType()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Function_definition_blockContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def single_function_definition(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Single_function_definitionContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Single_function_definitionContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_function_definition_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunction_definition_block" ):
                listener.enterFunction_definition_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunction_definition_block" ):
                listener.exitFunction_definition_block(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunction_definition_block" ):
                return visitor.visitFunction_definition_block(self)
            else:
                return visitor.visitChildren(self)




    def function_definition_block(self):

        localctx = fstripsParser.Function_definition_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_function_definition_block)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 280
            self.match(fstripsParser.T__0)
            self.state = 281
            self.match(fstripsParser.T__8)
            self.state = 285
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0:
                self.state = 282
                self.single_function_definition()
                self.state = 287
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 288
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Single_function_definitionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typed_function_definition(self):
            return self.getTypedRuleContext(fstripsParser.Typed_function_definitionContext,0)


        def untyped_function_definition(self):
            return self.getTypedRuleContext(fstripsParser.Untyped_function_definitionContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_single_function_definition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingle_function_definition" ):
                listener.enterSingle_function_definition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingle_function_definition" ):
                listener.exitSingle_function_definition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingle_function_definition" ):
                return visitor.visitSingle_function_definition(self)
            else:
                return visitor.visitChildren(self)




    def single_function_definition(self):

        localctx = fstripsParser.Single_function_definitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_single_function_definition)
        try:
            self.state = 292
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,28,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 290
                self.typed_function_definition()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 291
                self.untyped_function_definition()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Typed_function_definitionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def logical_symbol_name(self):
            return self.getTypedRuleContext(fstripsParser.Logical_symbol_nameContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def primitive_type(self):
            return self.getTypedRuleContext(fstripsParser.Primitive_typeContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_typed_function_definition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTyped_function_definition" ):
                listener.enterTyped_function_definition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTyped_function_definition" ):
                listener.exitTyped_function_definition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTyped_function_definition" ):
                return visitor.visitTyped_function_definition(self)
            else:
                return visitor.visitChildren(self)




    def typed_function_definition(self):

        localctx = fstripsParser.Typed_function_definitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_typed_function_definition)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 294
            self.match(fstripsParser.T__0)
            self.state = 295
            self.logical_symbol_name()
            self.state = 296
            self.possibly_typed_variable_list()
            self.state = 297
            self.match(fstripsParser.T__2)
            self.state = 298
            self.match(fstripsParser.T__6)
            self.state = 299
            self.primitive_type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Untyped_function_definitionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def logical_symbol_name(self):
            return self.getTypedRuleContext(fstripsParser.Logical_symbol_nameContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_untyped_function_definition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUntyped_function_definition" ):
                listener.enterUntyped_function_definition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUntyped_function_definition" ):
                listener.exitUntyped_function_definition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUntyped_function_definition" ):
                return visitor.visitUntyped_function_definition(self)
            else:
                return visitor.visitChildren(self)




    def untyped_function_definition(self):

        localctx = fstripsParser.Untyped_function_definitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_untyped_function_definition)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 301
            self.match(fstripsParser.T__0)
            self.state = 302
            self.logical_symbol_name()
            self.state = 303
            self.possibly_typed_variable_list()
            self.state = 304
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Logical_symbol_nameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def EXTNAME(self):
            return self.getToken(fstripsParser.EXTNAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_logical_symbol_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLogical_symbol_name" ):
                listener.enterLogical_symbol_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLogical_symbol_name" ):
                listener.exitLogical_symbol_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLogical_symbol_name" ):
                return visitor.visitLogical_symbol_name(self)
            else:
                return visitor.visitChildren(self)




    def logical_symbol_name(self):

        localctx = fstripsParser.Logical_symbol_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_logical_symbol_name)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 306
            _la = self._input.LA(1)
            if not(_la==fstripsParser.NAME or _la==fstripsParser.EXTNAME):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Constant_declarationContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def possibly_typed_name_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_name_listContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_constant_declaration

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConstant_declaration" ):
                listener.enterConstant_declaration(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConstant_declaration" ):
                listener.exitConstant_declaration(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConstant_declaration" ):
                return visitor.visitConstant_declaration(self)
            else:
                return visitor.visitChildren(self)




    def constant_declaration(self):

        localctx = fstripsParser.Constant_declarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_constant_declaration)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 308
            self.match(fstripsParser.T__0)
            self.state = 309
            self.match(fstripsParser.T__9)
            self.state = 310
            self.possibly_typed_name_list()
            self.state = 311
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Predicate_definition_blockContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def single_predicate_definition(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Single_predicate_definitionContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Single_predicate_definitionContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_predicate_definition_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPredicate_definition_block" ):
                listener.enterPredicate_definition_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPredicate_definition_block" ):
                listener.exitPredicate_definition_block(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPredicate_definition_block" ):
                return visitor.visitPredicate_definition_block(self)
            else:
                return visitor.visitChildren(self)




    def predicate_definition_block(self):

        localctx = fstripsParser.Predicate_definition_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_predicate_definition_block)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 313
            self.match(fstripsParser.T__0)
            self.state = 314
            self.match(fstripsParser.T__10)
            self.state = 318
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0:
                self.state = 315
                self.single_predicate_definition()
                self.state = 320
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 321
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Single_predicate_definitionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def predicate(self):
            return self.getTypedRuleContext(fstripsParser.PredicateContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_single_predicate_definition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingle_predicate_definition" ):
                listener.enterSingle_predicate_definition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingle_predicate_definition" ):
                listener.exitSingle_predicate_definition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingle_predicate_definition" ):
                return visitor.visitSingle_predicate_definition(self)
            else:
                return visitor.visitChildren(self)




    def single_predicate_definition(self):

        localctx = fstripsParser.Single_predicate_definitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_single_predicate_definition)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 323
            self.match(fstripsParser.T__0)
            self.state = 324
            self.predicate()
            self.state = 325
            self.possibly_typed_variable_list()
            self.state = 326
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class PredicateContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_predicate

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPredicate" ):
                listener.enterPredicate(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPredicate" ):
                listener.exitPredicate(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPredicate" ):
                return visitor.visitPredicate(self)
            else:
                return visitor.visitChildren(self)




    def predicate(self):

        localctx = fstripsParser.PredicateContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_predicate)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 328
            self.match(fstripsParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Function_nameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_function_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunction_name" ):
                listener.enterFunction_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunction_name" ):
                listener.exitFunction_name(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunction_name" ):
                return visitor.visitFunction_name(self)
            else:
                return visitor.visitChildren(self)




    def function_name(self):

        localctx = fstripsParser.Function_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_function_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 330
            self.match(fstripsParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class StructureDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def actionDef(self):
            return self.getTypedRuleContext(fstripsParser.ActionDefContext,0)


        def eventDef(self):
            return self.getTypedRuleContext(fstripsParser.EventDefContext,0)


        def derivedDef(self):
            return self.getTypedRuleContext(fstripsParser.DerivedDefContext,0)


        def constraintDef(self):
            return self.getTypedRuleContext(fstripsParser.ConstraintDefContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_structureDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStructureDef" ):
                listener.enterStructureDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStructureDef" ):
                listener.exitStructureDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStructureDef" ):
                return visitor.visitStructureDef(self)
            else:
                return visitor.visitChildren(self)




    def structureDef(self):

        localctx = fstripsParser.StructureDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_structureDef)
        try:
            self.state = 336
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,30,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 332
                self.actionDef()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 333
                self.eventDef()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 334
                self.derivedDef()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 335
                self.constraintDef()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ActionDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def K_ACTION(self):
            return self.getToken(fstripsParser.K_ACTION, 0)

        def actionName(self):
            return self.getTypedRuleContext(fstripsParser.ActionNameContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def actionDefBody(self):
            return self.getTypedRuleContext(fstripsParser.ActionDefBodyContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_actionDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterActionDef" ):
                listener.enterActionDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitActionDef" ):
                listener.exitActionDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitActionDef" ):
                return visitor.visitActionDef(self)
            else:
                return visitor.visitChildren(self)




    def actionDef(self):

        localctx = fstripsParser.ActionDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_actionDef)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 338
            self.match(fstripsParser.T__0)
            self.state = 339
            self.match(fstripsParser.K_ACTION)
            self.state = 340
            self.actionName()
            self.state = 341
            self.match(fstripsParser.T__11)
            self.state = 342
            self.match(fstripsParser.T__0)
            self.state = 343
            self.possibly_typed_variable_list()
            self.state = 344
            self.match(fstripsParser.T__2)
            self.state = 345
            self.actionDefBody()
            self.state = 346
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ConstraintDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def constraintSymbol(self):
            return self.getTypedRuleContext(fstripsParser.ConstraintSymbolContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_constraintDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConstraintDef" ):
                listener.enterConstraintDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConstraintDef" ):
                listener.exitConstraintDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConstraintDef" ):
                return visitor.visitConstraintDef(self)
            else:
                return visitor.visitChildren(self)




    def constraintDef(self):

        localctx = fstripsParser.ConstraintDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_constraintDef)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 348
            self.match(fstripsParser.T__0)
            self.state = 349
            self.match(fstripsParser.T__12)
            self.state = 350
            self.constraintSymbol()
            self.state = 351
            self.match(fstripsParser.T__11)
            self.state = 352
            self.match(fstripsParser.T__0)
            self.state = 353
            self.possibly_typed_variable_list()
            self.state = 354
            self.match(fstripsParser.T__2)
            self.state = 355
            self.match(fstripsParser.T__13)
            self.state = 356
            self.goalDesc()
            self.state = 357
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class EventDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def eventSymbol(self):
            return self.getTypedRuleContext(fstripsParser.EventSymbolContext,0)


        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def actionDefBody(self):
            return self.getTypedRuleContext(fstripsParser.ActionDefBodyContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_eventDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEventDef" ):
                listener.enterEventDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEventDef" ):
                listener.exitEventDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEventDef" ):
                return visitor.visitEventDef(self)
            else:
                return visitor.visitChildren(self)




    def eventDef(self):

        localctx = fstripsParser.EventDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_eventDef)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 359
            self.match(fstripsParser.T__0)
            self.state = 360
            self.match(fstripsParser.T__14)
            self.state = 361
            self.eventSymbol()
            self.state = 362
            self.match(fstripsParser.T__11)
            self.state = 363
            self.match(fstripsParser.T__0)
            self.state = 364
            self.possibly_typed_variable_list()
            self.state = 365
            self.match(fstripsParser.T__2)
            self.state = 366
            self.actionDefBody()
            self.state = 367
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ActionNameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def EXTNAME(self):
            return self.getToken(fstripsParser.EXTNAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_actionName

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterActionName" ):
                listener.enterActionName(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitActionName" ):
                listener.exitActionName(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitActionName" ):
                return visitor.visitActionName(self)
            else:
                return visitor.visitChildren(self)




    def actionName(self):

        localctx = fstripsParser.ActionNameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_actionName)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 369
            _la = self._input.LA(1)
            if not(_la==fstripsParser.NAME or _la==fstripsParser.EXTNAME):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ConstraintSymbolContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_constraintSymbol

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConstraintSymbol" ):
                listener.enterConstraintSymbol(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConstraintSymbol" ):
                listener.exitConstraintSymbol(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConstraintSymbol" ):
                return visitor.visitConstraintSymbol(self)
            else:
                return visitor.visitChildren(self)




    def constraintSymbol(self):

        localctx = fstripsParser.ConstraintSymbolContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_constraintSymbol)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 371
            self.match(fstripsParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class EventSymbolContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def EXTNAME(self):
            return self.getToken(fstripsParser.EXTNAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_eventSymbol

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEventSymbol" ):
                listener.enterEventSymbol(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEventSymbol" ):
                listener.exitEventSymbol(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEventSymbol" ):
                return visitor.visitEventSymbol(self)
            else:
                return visitor.visitChildren(self)




    def eventSymbol(self):

        localctx = fstripsParser.EventSymbolContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_eventSymbol)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 373
            _la = self._input.LA(1)
            if not(_la==fstripsParser.NAME or _la==fstripsParser.EXTNAME):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ActionDefBodyContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def K_PRECONDITION(self):
            return self.getToken(fstripsParser.K_PRECONDITION, 0)

        def precondition(self):
            return self.getTypedRuleContext(fstripsParser.PreconditionContext,0)


        def K_EFFECT(self):
            return self.getToken(fstripsParser.K_EFFECT, 0)

        def effect(self):
            return self.getTypedRuleContext(fstripsParser.EffectContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_actionDefBody

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterActionDefBody" ):
                listener.enterActionDefBody(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitActionDefBody" ):
                listener.exitActionDefBody(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitActionDefBody" ):
                return visitor.visitActionDefBody(self)
            else:
                return visitor.visitChildren(self)




    def actionDefBody(self):

        localctx = fstripsParser.ActionDefBodyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_actionDefBody)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 375
            self.match(fstripsParser.K_PRECONDITION)
            self.state = 376
            self.precondition()
            self.state = 377
            self.match(fstripsParser.K_EFFECT)
            self.state = 378
            self.effect()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class PreconditionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_precondition

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class RegularPreconditionContext(PreconditionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PreconditionContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRegularPrecondition" ):
                listener.enterRegularPrecondition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRegularPrecondition" ):
                listener.exitRegularPrecondition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitRegularPrecondition" ):
                return visitor.visitRegularPrecondition(self)
            else:
                return visitor.visitChildren(self)


    class TrivialPreconditionContext(PreconditionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PreconditionContext
            super().__init__(parser)
            self.copyFrom(ctx)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTrivialPrecondition" ):
                listener.enterTrivialPrecondition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTrivialPrecondition" ):
                listener.exitTrivialPrecondition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTrivialPrecondition" ):
                return visitor.visitTrivialPrecondition(self)
            else:
                return visitor.visitChildren(self)



    def precondition(self):

        localctx = fstripsParser.PreconditionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_precondition)
        try:
            self.state = 383
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,31,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.TrivialPreconditionContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 380
                self.match(fstripsParser.T__0)
                self.state = 381
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.RegularPreconditionContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 382
                self.goalDesc()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class GoalDescContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_goalDesc

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class BuiltinBinaryAtomContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def builtin_binary_predicate(self):
            return self.getTypedRuleContext(fstripsParser.Builtin_binary_predicateContext,0)

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.TermContext)
            else:
                return self.getTypedRuleContext(fstripsParser.TermContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBuiltinBinaryAtom" ):
                listener.enterBuiltinBinaryAtom(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBuiltinBinaryAtom" ):
                listener.exitBuiltinBinaryAtom(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBuiltinBinaryAtom" ):
                return visitor.visitBuiltinBinaryAtom(self)
            else:
                return visitor.visitChildren(self)


    class AndGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_AND(self):
            return self.getToken(fstripsParser.K_AND, 0)
        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAndGoalDesc" ):
                listener.enterAndGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAndGoalDesc" ):
                listener.exitAndGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAndGoalDesc" ):
                return visitor.visitAndGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class OrGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_OR(self):
            return self.getToken(fstripsParser.K_OR, 0)
        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOrGoalDesc" ):
                listener.enterOrGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOrGoalDesc" ):
                listener.exitOrGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOrGoalDesc" ):
                return visitor.visitOrGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class UniversalGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_FORALL(self):
            return self.getToken(fstripsParser.K_FORALL, 0)
        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUniversalGoalDesc" ):
                listener.enterUniversalGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUniversalGoalDesc" ):
                listener.exitUniversalGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUniversalGoalDesc" ):
                return visitor.visitUniversalGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class TermGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def atomicTermFormula(self):
            return self.getTypedRuleContext(fstripsParser.AtomicTermFormulaContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermGoalDesc" ):
                listener.enterTermGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermGoalDesc" ):
                listener.exitTermGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermGoalDesc" ):
                return visitor.visitTermGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class ExistentialGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_EXISTS(self):
            return self.getToken(fstripsParser.K_EXISTS, 0)
        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExistentialGoalDesc" ):
                listener.enterExistentialGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExistentialGoalDesc" ):
                listener.exitExistentialGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExistentialGoalDesc" ):
                return visitor.visitExistentialGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class NotGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_NOT(self):
            return self.getToken(fstripsParser.K_NOT, 0)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNotGoalDesc" ):
                listener.enterNotGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNotGoalDesc" ):
                listener.exitNotGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNotGoalDesc" ):
                return visitor.visitNotGoalDesc(self)
            else:
                return visitor.visitChildren(self)


    class ImplyGoalDescContext(GoalDescContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.GoalDescContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_IMPLY(self):
            return self.getToken(fstripsParser.K_IMPLY, 0)
        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterImplyGoalDesc" ):
                listener.enterImplyGoalDesc(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitImplyGoalDesc" ):
                listener.exitImplyGoalDesc(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitImplyGoalDesc" ):
                return visitor.visitImplyGoalDesc(self)
            else:
                return visitor.visitChildren(self)



    def goalDesc(self):

        localctx = fstripsParser.GoalDescContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_goalDesc)
        self._la = 0 # Token type
        try:
            self.state = 437
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,34,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.TermGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 385
                self.atomicTermFormula()
                pass

            elif la_ == 2:
                localctx = fstripsParser.AndGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 386
                self.match(fstripsParser.T__0)
                self.state = 387
                self.match(fstripsParser.K_AND)
                self.state = 391
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0:
                    self.state = 388
                    self.goalDesc()
                    self.state = 393
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 394
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.OrGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 395
                self.match(fstripsParser.T__0)
                self.state = 396
                self.match(fstripsParser.K_OR)
                self.state = 400
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0:
                    self.state = 397
                    self.goalDesc()
                    self.state = 402
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 403
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 4:
                localctx = fstripsParser.NotGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 404
                self.match(fstripsParser.T__0)
                self.state = 405
                self.match(fstripsParser.K_NOT)
                self.state = 406
                self.goalDesc()
                self.state = 407
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 5:
                localctx = fstripsParser.ImplyGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 5)
                self.state = 409
                self.match(fstripsParser.T__0)
                self.state = 410
                self.match(fstripsParser.K_IMPLY)
                self.state = 411
                self.goalDesc()
                self.state = 412
                self.goalDesc()
                self.state = 413
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 6:
                localctx = fstripsParser.ExistentialGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 6)
                self.state = 415
                self.match(fstripsParser.T__0)
                self.state = 416
                self.match(fstripsParser.K_EXISTS)
                self.state = 417
                self.match(fstripsParser.T__0)
                self.state = 418
                self.possibly_typed_variable_list()
                self.state = 419
                self.match(fstripsParser.T__2)
                self.state = 420
                self.goalDesc()
                self.state = 421
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 7:
                localctx = fstripsParser.UniversalGoalDescContext(self, localctx)
                self.enterOuterAlt(localctx, 7)
                self.state = 423
                self.match(fstripsParser.T__0)
                self.state = 424
                self.match(fstripsParser.K_FORALL)
                self.state = 425
                self.match(fstripsParser.T__0)
                self.state = 426
                self.possibly_typed_variable_list()
                self.state = 427
                self.match(fstripsParser.T__2)
                self.state = 428
                self.goalDesc()
                self.state = 429
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 8:
                localctx = fstripsParser.BuiltinBinaryAtomContext(self, localctx)
                self.enterOuterAlt(localctx, 8)
                self.state = 431
                self.match(fstripsParser.T__0)
                self.state = 432
                self.builtin_binary_predicate()
                self.state = 433
                self.term()
                self.state = 434
                self.term()
                self.state = 435
                self.match(fstripsParser.T__2)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class AtomicTermFormulaContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def predicate(self):
            return self.getTypedRuleContext(fstripsParser.PredicateContext,0)


        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.TermContext)
            else:
                return self.getTypedRuleContext(fstripsParser.TermContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_atomicTermFormula

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtomicTermFormula" ):
                listener.enterAtomicTermFormula(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtomicTermFormula" ):
                listener.exitAtomicTermFormula(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtomicTermFormula" ):
                return visitor.visitAtomicTermFormula(self)
            else:
                return visitor.visitChildren(self)




    def atomicTermFormula(self):

        localctx = fstripsParser.AtomicTermFormulaContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_atomicTermFormula)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 439
            self.match(fstripsParser.T__0)
            self.state = 440
            self.predicate()
            self.state = 444
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0 or _la==fstripsParser.T__15 or ((((_la - 83)) & ~0x3f) == 0 and ((1 << (_la - 83)) & ((1 << (fstripsParser.NAME - 83)) | (1 << (fstripsParser.VARIABLE - 83)) | (1 << (fstripsParser.NUMBER - 83)))) != 0):
                self.state = 441
                self.term()
                self.state = 446
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 447
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class TermContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_term

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class TermObjectContext(TermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermObject" ):
                listener.enterTermObject(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermObject" ):
                listener.exitTermObject(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermObject" ):
                return visitor.visitTermObject(self)
            else:
                return visitor.visitChildren(self)


    class TermTimeStepContext(TermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TermContext
            super().__init__(parser)
            self.copyFrom(ctx)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermTimeStep" ):
                listener.enterTermTimeStep(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermTimeStep" ):
                listener.exitTermTimeStep(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermTimeStep" ):
                return visitor.visitTermTimeStep(self)
            else:
                return visitor.visitChildren(self)


    class TermFunctionContext(TermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermFunction" ):
                listener.enterTermFunction(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermFunction" ):
                listener.exitTermFunction(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermFunction" ):
                return visitor.visitTermFunction(self)
            else:
                return visitor.visitChildren(self)


    class TermVariableContext(TermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def VARIABLE(self):
            return self.getToken(fstripsParser.VARIABLE, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermVariable" ):
                listener.enterTermVariable(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermVariable" ):
                listener.exitTermVariable(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermVariable" ):
                return visitor.visitTermVariable(self)
            else:
                return visitor.visitChildren(self)


    class TermNumberContext(TermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.TermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(fstripsParser.NUMBER, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTermNumber" ):
                listener.enterTermNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTermNumber" ):
                listener.exitTermNumber(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTermNumber" ):
                return visitor.visitTermNumber(self)
            else:
                return visitor.visitChildren(self)



    def term(self):

        localctx = fstripsParser.TermContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_term)
        try:
            self.state = 454
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.NAME]:
                localctx = fstripsParser.TermObjectContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 449
                self.match(fstripsParser.NAME)
                pass
            elif token in [fstripsParser.NUMBER]:
                localctx = fstripsParser.TermNumberContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 450
                self.match(fstripsParser.NUMBER)
                pass
            elif token in [fstripsParser.VARIABLE]:
                localctx = fstripsParser.TermVariableContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 451
                self.match(fstripsParser.VARIABLE)
                pass
            elif token in [fstripsParser.T__15]:
                localctx = fstripsParser.TermTimeStepContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 452
                self.match(fstripsParser.T__15)
                pass
            elif token in [fstripsParser.T__0]:
                localctx = fstripsParser.TermFunctionContext(self, localctx)
                self.enterOuterAlt(localctx, 5)
                self.state = 453
                self.functionTerm()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class FunctionTermContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_functionTerm

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class UnaryArithmeticFunctionTermContext(FunctionTermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.FunctionTermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def builtin_unary_function(self):
            return self.getTypedRuleContext(fstripsParser.Builtin_unary_functionContext,0)

        def term(self):
            return self.getTypedRuleContext(fstripsParser.TermContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUnaryArithmeticFunctionTerm" ):
                listener.enterUnaryArithmeticFunctionTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUnaryArithmeticFunctionTerm" ):
                listener.exitUnaryArithmeticFunctionTerm(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUnaryArithmeticFunctionTerm" ):
                return visitor.visitUnaryArithmeticFunctionTerm(self)
            else:
                return visitor.visitChildren(self)


    class BinaryArithmeticFunctionTermContext(FunctionTermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.FunctionTermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def builtin_binary_function(self):
            return self.getTypedRuleContext(fstripsParser.Builtin_binary_functionContext,0)

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.TermContext)
            else:
                return self.getTypedRuleContext(fstripsParser.TermContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBinaryArithmeticFunctionTerm" ):
                listener.enterBinaryArithmeticFunctionTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBinaryArithmeticFunctionTerm" ):
                listener.exitBinaryArithmeticFunctionTerm(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBinaryArithmeticFunctionTerm" ):
                return visitor.visitBinaryArithmeticFunctionTerm(self)
            else:
                return visitor.visitChildren(self)


    class GenericFunctionTermContext(FunctionTermContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.FunctionTermContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def logical_symbol_name(self):
            return self.getTypedRuleContext(fstripsParser.Logical_symbol_nameContext,0)

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.TermContext)
            else:
                return self.getTypedRuleContext(fstripsParser.TermContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGenericFunctionTerm" ):
                listener.enterGenericFunctionTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGenericFunctionTerm" ):
                listener.exitGenericFunctionTerm(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitGenericFunctionTerm" ):
                return visitor.visitGenericFunctionTerm(self)
            else:
                return visitor.visitChildren(self)



    def functionTerm(self):

        localctx = fstripsParser.FunctionTermContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_functionTerm)
        self._la = 0 # Token type
        try:
            self.state = 477
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,38,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.GenericFunctionTermContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 456
                self.match(fstripsParser.T__0)
                self.state = 457
                self.logical_symbol_name()
                self.state = 461
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0 or _la==fstripsParser.T__15 or ((((_la - 83)) & ~0x3f) == 0 and ((1 << (_la - 83)) & ((1 << (fstripsParser.NAME - 83)) | (1 << (fstripsParser.VARIABLE - 83)) | (1 << (fstripsParser.NUMBER - 83)))) != 0):
                    self.state = 458
                    self.term()
                    self.state = 463
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 464
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.BinaryArithmeticFunctionTermContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 466
                self.match(fstripsParser.T__0)
                self.state = 467
                self.builtin_binary_function()
                self.state = 468
                self.term()
                self.state = 469
                self.term()
                self.state = 470
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.UnaryArithmeticFunctionTermContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 472
                self.match(fstripsParser.T__0)
                self.state = 473
                self.builtin_unary_function()
                self.state = 474
                self.term()
                self.state = 475
                self.match(fstripsParser.T__2)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class DerivedDefContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)


        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_derivedDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDerivedDef" ):
                listener.enterDerivedDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDerivedDef" ):
                listener.exitDerivedDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDerivedDef" ):
                return visitor.visitDerivedDef(self)
            else:
                return visitor.visitChildren(self)




    def derivedDef(self):

        localctx = fstripsParser.DerivedDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_derivedDef)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 479
            self.match(fstripsParser.T__0)
            self.state = 480
            self.match(fstripsParser.T__16)
            self.state = 481
            self.match(fstripsParser.T__0)
            self.state = 482
            self.match(fstripsParser.NAME)
            self.state = 483
            self.possibly_typed_variable_list()
            self.state = 484
            self.match(fstripsParser.T__2)
            self.state = 485
            self.goalDesc()
            self.state = 486
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class EffectContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_effect

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class SingleEffectContext(EffectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.EffectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def single_effect(self):
            return self.getTypedRuleContext(fstripsParser.Single_effectContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleEffect" ):
                listener.enterSingleEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleEffect" ):
                listener.exitSingleEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingleEffect" ):
                return visitor.visitSingleEffect(self)
            else:
                return visitor.visitChildren(self)


    class ConjunctiveEffectFormulaContext(EffectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.EffectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_AND(self):
            return self.getToken(fstripsParser.K_AND, 0)
        def single_effect(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Single_effectContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Single_effectContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConjunctiveEffectFormula" ):
                listener.enterConjunctiveEffectFormula(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConjunctiveEffectFormula" ):
                listener.exitConjunctiveEffectFormula(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConjunctiveEffectFormula" ):
                return visitor.visitConjunctiveEffectFormula(self)
            else:
                return visitor.visitChildren(self)



    def effect(self):

        localctx = fstripsParser.EffectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_effect)
        self._la = 0 # Token type
        try:
            self.state = 498
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,40,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.ConjunctiveEffectFormulaContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 488
                self.match(fstripsParser.T__0)
                self.state = 489
                self.match(fstripsParser.K_AND)
                self.state = 493
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0:
                    self.state = 490
                    self.single_effect()
                    self.state = 495
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 496
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.SingleEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 497
                self.single_effect()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Single_effectContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_single_effect

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class SingleConditionalEffectContext(Single_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Single_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_WHEN(self):
            return self.getToken(fstripsParser.K_WHEN, 0)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)

        def atomic_effect(self):
            return self.getTypedRuleContext(fstripsParser.Atomic_effectContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingleConditionalEffect" ):
                listener.enterSingleConditionalEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingleConditionalEffect" ):
                listener.exitSingleConditionalEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingleConditionalEffect" ):
                return visitor.visitSingleConditionalEffect(self)
            else:
                return visitor.visitChildren(self)


    class MultipleConditionalEffectContext(Single_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Single_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_WHEN(self):
            return self.getToken(fstripsParser.K_WHEN, 0)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)

        def K_AND(self):
            return self.getToken(fstripsParser.K_AND, 0)
        def atomic_effect(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Atomic_effectContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Atomic_effectContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMultipleConditionalEffect" ):
                listener.enterMultipleConditionalEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMultipleConditionalEffect" ):
                listener.exitMultipleConditionalEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMultipleConditionalEffect" ):
                return visitor.visitMultipleConditionalEffect(self)
            else:
                return visitor.visitChildren(self)


    class UniversallyQuantifiedEffectContext(Single_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Single_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_FORALL(self):
            return self.getToken(fstripsParser.K_FORALL, 0)
        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)

        def effect(self):
            return self.getTypedRuleContext(fstripsParser.EffectContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUniversallyQuantifiedEffect" ):
                listener.enterUniversallyQuantifiedEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUniversallyQuantifiedEffect" ):
                listener.exitUniversallyQuantifiedEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUniversallyQuantifiedEffect" ):
                return visitor.visitUniversallyQuantifiedEffect(self)
            else:
                return visitor.visitChildren(self)


    class AtomicEffectContext(Single_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Single_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def atomic_effect(self):
            return self.getTypedRuleContext(fstripsParser.Atomic_effectContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtomicEffect" ):
                listener.enterAtomicEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtomicEffect" ):
                listener.exitAtomicEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtomicEffect" ):
                return visitor.visitAtomicEffect(self)
            else:
                return visitor.visitChildren(self)



    def single_effect(self):

        localctx = fstripsParser.Single_effectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_single_effect)
        self._la = 0 # Token type
        try:
            self.state = 529
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,42,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.UniversallyQuantifiedEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 500
                self.match(fstripsParser.T__0)
                self.state = 501
                self.match(fstripsParser.K_FORALL)
                self.state = 502
                self.match(fstripsParser.T__0)
                self.state = 503
                self.possibly_typed_variable_list()
                self.state = 504
                self.match(fstripsParser.T__2)
                self.state = 505
                self.effect()
                self.state = 506
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.SingleConditionalEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 508
                self.match(fstripsParser.T__0)
                self.state = 509
                self.match(fstripsParser.K_WHEN)
                self.state = 510
                self.goalDesc()
                self.state = 511
                self.atomic_effect()
                self.state = 512
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.MultipleConditionalEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 514
                self.match(fstripsParser.T__0)
                self.state = 515
                self.match(fstripsParser.K_WHEN)
                self.state = 516
                self.goalDesc()
                self.state = 517
                self.match(fstripsParser.T__0)
                self.state = 518
                self.match(fstripsParser.K_AND)
                self.state = 522
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0:
                    self.state = 519
                    self.atomic_effect()
                    self.state = 524
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 525
                self.match(fstripsParser.T__2)
                self.state = 526
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 4:
                localctx = fstripsParser.AtomicEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 528
                self.atomic_effect()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Atomic_effectContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_atomic_effect

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class DeleteAtomEffectContext(Atomic_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Atomic_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_NOT(self):
            return self.getToken(fstripsParser.K_NOT, 0)
        def atomicTermFormula(self):
            return self.getTypedRuleContext(fstripsParser.AtomicTermFormulaContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDeleteAtomEffect" ):
                listener.enterDeleteAtomEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDeleteAtomEffect" ):
                listener.exitDeleteAtomEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDeleteAtomEffect" ):
                return visitor.visitDeleteAtomEffect(self)
            else:
                return visitor.visitChildren(self)


    class AssignEffectContext(Atomic_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Atomic_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def assignOp(self):
            return self.getTypedRuleContext(fstripsParser.AssignOpContext,0)

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)

        def term(self):
            return self.getTypedRuleContext(fstripsParser.TermContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssignEffect" ):
                listener.enterAssignEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssignEffect" ):
                listener.exitAssignEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAssignEffect" ):
                return visitor.visitAssignEffect(self)
            else:
                return visitor.visitChildren(self)


    class AssignConstantContext(Atomic_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Atomic_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)

        def term(self):
            return self.getTypedRuleContext(fstripsParser.TermContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssignConstant" ):
                listener.enterAssignConstant(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssignConstant" ):
                listener.exitAssignConstant(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAssignConstant" ):
                return visitor.visitAssignConstant(self)
            else:
                return visitor.visitChildren(self)


    class AddAtomEffectContext(Atomic_effectContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Atomic_effectContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def atomicTermFormula(self):
            return self.getTypedRuleContext(fstripsParser.AtomicTermFormulaContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAddAtomEffect" ):
                listener.enterAddAtomEffect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAddAtomEffect" ):
                listener.exitAddAtomEffect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAddAtomEffect" ):
                return visitor.visitAddAtomEffect(self)
            else:
                return visitor.visitChildren(self)



    def atomic_effect(self):

        localctx = fstripsParser.Atomic_effectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_atomic_effect)
        try:
            self.state = 549
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,43,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.AssignConstantContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 531
                self.match(fstripsParser.T__0)
                self.state = 532
                self.match(fstripsParser.T__17)
                self.state = 533
                self.functionTerm()
                self.state = 534
                self.term()
                self.state = 535
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.AssignEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 537
                self.match(fstripsParser.T__0)
                self.state = 538
                self.assignOp()
                self.state = 539
                self.functionTerm()
                self.state = 540
                self.term()
                self.state = 541
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.DeleteAtomEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 543
                self.match(fstripsParser.T__0)
                self.state = 544
                self.match(fstripsParser.K_NOT)
                self.state = 545
                self.atomicTermFormula()
                self.state = 546
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 4:
                localctx = fstripsParser.AddAtomEffectContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 548
                self.atomicTermFormula()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Builtin_binary_functionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_builtin_binary_function

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBuiltin_binary_function" ):
                listener.enterBuiltin_binary_function(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBuiltin_binary_function" ):
                listener.exitBuiltin_binary_function(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBuiltin_binary_function" ):
                return visitor.visitBuiltin_binary_function(self)
            else:
                return visitor.visitChildren(self)




    def builtin_binary_function(self):

        localctx = fstripsParser.Builtin_binary_functionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_builtin_binary_function)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 551
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << fstripsParser.T__6) | (1 << fstripsParser.T__18) | (1 << fstripsParser.T__19) | (1 << fstripsParser.T__20) | (1 << fstripsParser.T__21) | (1 << fstripsParser.T__22) | (1 << fstripsParser.T__23))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Builtin_unary_functionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_builtin_unary_function

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBuiltin_unary_function" ):
                listener.enterBuiltin_unary_function(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBuiltin_unary_function" ):
                listener.exitBuiltin_unary_function(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBuiltin_unary_function" ):
                return visitor.visitBuiltin_unary_function(self)
            else:
                return visitor.visitChildren(self)




    def builtin_unary_function(self):

        localctx = fstripsParser.Builtin_unary_functionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_builtin_unary_function)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 553
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << fstripsParser.T__6) | (1 << fstripsParser.T__24) | (1 << fstripsParser.T__25) | (1 << fstripsParser.T__26) | (1 << fstripsParser.T__27) | (1 << fstripsParser.T__28) | (1 << fstripsParser.T__29) | (1 << fstripsParser.T__30) | (1 << fstripsParser.T__31) | (1 << fstripsParser.T__32))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Builtin_binary_predicateContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_builtin_binary_predicate

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBuiltin_binary_predicate" ):
                listener.enterBuiltin_binary_predicate(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBuiltin_binary_predicate" ):
                listener.exitBuiltin_binary_predicate(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBuiltin_binary_predicate" ):
                return visitor.visitBuiltin_binary_predicate(self)
            else:
                return visitor.visitChildren(self)




    def builtin_binary_predicate(self):

        localctx = fstripsParser.Builtin_binary_predicateContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_builtin_binary_predicate)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 555
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << fstripsParser.T__33) | (1 << fstripsParser.T__34) | (1 << fstripsParser.T__35) | (1 << fstripsParser.T__36) | (1 << fstripsParser.T__37))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class AssignOpContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def K_SCALEUP(self):
            return self.getToken(fstripsParser.K_SCALEUP, 0)

        def K_SCALEDOWN(self):
            return self.getToken(fstripsParser.K_SCALEDOWN, 0)

        def K_INCREASE(self):
            return self.getToken(fstripsParser.K_INCREASE, 0)

        def K_DECREASE(self):
            return self.getToken(fstripsParser.K_DECREASE, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_assignOp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssignOp" ):
                listener.enterAssignOp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssignOp" ):
                listener.exitAssignOp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAssignOp" ):
                return visitor.visitAssignOp(self)
            else:
                return visitor.visitChildren(self)




    def assignOp(self):

        localctx = fstripsParser.AssignOpContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_assignOp)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 557
            _la = self._input.LA(1)
            if not(((((_la - 75)) & ~0x3f) == 0 and ((1 << (_la - 75)) & ((1 << (fstripsParser.K_INCREASE - 75)) | (1 << (fstripsParser.K_DECREASE - 75)) | (1 << (fstripsParser.K_SCALEUP - 75)) | (1 << (fstripsParser.K_SCALEDOWN - 75)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ProblemContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def problemDecl(self):
            return self.getTypedRuleContext(fstripsParser.ProblemDeclContext,0)


        def problemDomain(self):
            return self.getTypedRuleContext(fstripsParser.ProblemDomainContext,0)


        def init(self):
            return self.getTypedRuleContext(fstripsParser.InitContext,0)


        def goal(self):
            return self.getTypedRuleContext(fstripsParser.GoalContext,0)


        def requireDef(self):
            return self.getTypedRuleContext(fstripsParser.RequireDefContext,0)


        def object_declaration(self):
            return self.getTypedRuleContext(fstripsParser.Object_declarationContext,0)


        def problemMeta(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.ProblemMetaContext)
            else:
                return self.getTypedRuleContext(fstripsParser.ProblemMetaContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_problem

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProblem" ):
                listener.enterProblem(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProblem" ):
                listener.exitProblem(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProblem" ):
                return visitor.visitProblem(self)
            else:
                return visitor.visitChildren(self)




    def problem(self):

        localctx = fstripsParser.ProblemContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_problem)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 559
            self.match(fstripsParser.T__0)
            self.state = 560
            self.match(fstripsParser.T__1)
            self.state = 561
            self.problemDecl()
            self.state = 562
            self.problemDomain()
            self.state = 564
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,44,self._ctx)
            if la_ == 1:
                self.state = 563
                self.requireDef()


            self.state = 567
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,45,self._ctx)
            if la_ == 1:
                self.state = 566
                self.object_declaration()


            self.state = 569
            self.init()
            self.state = 570
            self.goal()
            self.state = 574
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0:
                self.state = 571
                self.problemMeta()
                self.state = 576
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 577
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ProblemMetaContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def probConstraints(self):
            return self.getTypedRuleContext(fstripsParser.ProbConstraintsContext,0)


        def boundsDecl(self):
            return self.getTypedRuleContext(fstripsParser.BoundsDeclContext,0)


        def metricSpec(self):
            return self.getTypedRuleContext(fstripsParser.MetricSpecContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_problemMeta

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProblemMeta" ):
                listener.enterProblemMeta(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProblemMeta" ):
                listener.exitProblemMeta(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProblemMeta" ):
                return visitor.visitProblemMeta(self)
            else:
                return visitor.visitChildren(self)




    def problemMeta(self):

        localctx = fstripsParser.ProblemMetaContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_problemMeta)
        try:
            self.state = 582
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,47,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 579
                self.probConstraints()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 580
                self.boundsDecl()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 581
                self.metricSpec()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ProblemDeclContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_problemDecl

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProblemDecl" ):
                listener.enterProblemDecl(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProblemDecl" ):
                listener.exitProblemDecl(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProblemDecl" ):
                return visitor.visitProblemDecl(self)
            else:
                return visitor.visitChildren(self)




    def problemDecl(self):

        localctx = fstripsParser.ProblemDeclContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_problemDecl)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 584
            self.match(fstripsParser.T__0)
            self.state = 585
            self.match(fstripsParser.T__38)
            self.state = 586
            self.match(fstripsParser.NAME)
            self.state = 587
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ProblemDomainContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def getRuleIndex(self):
            return fstripsParser.RULE_problemDomain

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProblemDomain" ):
                listener.enterProblemDomain(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProblemDomain" ):
                listener.exitProblemDomain(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProblemDomain" ):
                return visitor.visitProblemDomain(self)
            else:
                return visitor.visitChildren(self)




    def problemDomain(self):

        localctx = fstripsParser.ProblemDomainContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_problemDomain)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 589
            self.match(fstripsParser.T__0)
            self.state = 590
            self.match(fstripsParser.T__39)
            self.state = 591
            self.match(fstripsParser.NAME)
            self.state = 592
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Object_declarationContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def possibly_typed_name_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_name_listContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_object_declaration

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterObject_declaration" ):
                listener.enterObject_declaration(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitObject_declaration" ):
                listener.exitObject_declaration(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitObject_declaration" ):
                return visitor.visitObject_declaration(self)
            else:
                return visitor.visitChildren(self)




    def object_declaration(self):

        localctx = fstripsParser.Object_declarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 98, self.RULE_object_declaration)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 594
            self.match(fstripsParser.T__0)
            self.state = 595
            self.match(fstripsParser.T__40)
            self.state = 596
            self.possibly_typed_name_list()
            self.state = 597
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class BoundsDeclContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def typeBoundsDefinition(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.TypeBoundsDefinitionContext)
            else:
                return self.getTypedRuleContext(fstripsParser.TypeBoundsDefinitionContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_boundsDecl

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBoundsDecl" ):
                listener.enterBoundsDecl(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBoundsDecl" ):
                listener.exitBoundsDecl(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBoundsDecl" ):
                return visitor.visitBoundsDecl(self)
            else:
                return visitor.visitChildren(self)




    def boundsDecl(self):

        localctx = fstripsParser.BoundsDeclContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_boundsDecl)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 599
            self.match(fstripsParser.T__0)
            self.state = 600
            self.match(fstripsParser.T__41)
            self.state = 602 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 601
                self.typeBoundsDefinition()
                self.state = 604 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==fstripsParser.T__0):
                    break

            self.state = 606
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class TypeBoundsDefinitionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def numericBuiltinType(self):
            return self.getTypedRuleContext(fstripsParser.NumericBuiltinTypeContext,0)


        def NUMBER(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NUMBER)
            else:
                return self.getToken(fstripsParser.NUMBER, i)

        def getRuleIndex(self):
            return fstripsParser.RULE_typeBoundsDefinition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTypeBoundsDefinition" ):
                listener.enterTypeBoundsDefinition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTypeBoundsDefinition" ):
                listener.exitTypeBoundsDefinition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTypeBoundsDefinition" ):
                return visitor.visitTypeBoundsDefinition(self)
            else:
                return visitor.visitChildren(self)




    def typeBoundsDefinition(self):

        localctx = fstripsParser.TypeBoundsDefinitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_typeBoundsDefinition)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 608
            self.match(fstripsParser.T__0)
            self.state = 609
            self.match(fstripsParser.NAME)
            self.state = 610
            self.match(fstripsParser.T__6)
            self.state = 611
            self.numericBuiltinType()
            self.state = 612
            self.match(fstripsParser.T__42)
            self.state = 613
            self.match(fstripsParser.NUMBER)
            self.state = 614
            self.match(fstripsParser.T__43)
            self.state = 615
            self.match(fstripsParser.NUMBER)
            self.state = 616
            self.match(fstripsParser.T__44)
            self.state = 617
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class InitContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def K_INIT(self):
            return self.getToken(fstripsParser.K_INIT, 0)

        def init_element(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Init_elementContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Init_elementContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_init

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInit" ):
                listener.enterInit(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInit" ):
                listener.exitInit(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInit" ):
                return visitor.visitInit(self)
            else:
                return visitor.visitChildren(self)




    def init(self):

        localctx = fstripsParser.InitContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_init)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 619
            self.match(fstripsParser.T__0)
            self.state = 620
            self.match(fstripsParser.K_INIT)
            self.state = 624
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.T__0:
                self.state = 621
                self.init_element()
                self.state = 626
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 627
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Init_elementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_init_element

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class InitFunctionAssignmentContext(Init_elementContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Init_elementContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def flat_term(self):
            return self.getTypedRuleContext(fstripsParser.Flat_termContext,0)

        def constant_name(self):
            return self.getTypedRuleContext(fstripsParser.Constant_nameContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInitFunctionAssignment" ):
                listener.enterInitFunctionAssignment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInitFunctionAssignment" ):
                listener.exitInitFunctionAssignment(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInitFunctionAssignment" ):
                return visitor.visitInitFunctionAssignment(self)
            else:
                return visitor.visitChildren(self)


    class InitNegativeLiteralContext(Init_elementContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Init_elementContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_NOT(self):
            return self.getToken(fstripsParser.K_NOT, 0)
        def flat_atom(self):
            return self.getTypedRuleContext(fstripsParser.Flat_atomContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInitNegativeLiteral" ):
                listener.enterInitNegativeLiteral(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInitNegativeLiteral" ):
                listener.exitInitNegativeLiteral(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInitNegativeLiteral" ):
                return visitor.visitInitNegativeLiteral(self)
            else:
                return visitor.visitChildren(self)


    class InitPositiveLiteralContext(Init_elementContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Init_elementContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def flat_atom(self):
            return self.getTypedRuleContext(fstripsParser.Flat_atomContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInitPositiveLiteral" ):
                listener.enterInitPositiveLiteral(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInitPositiveLiteral" ):
                listener.exitInitPositiveLiteral(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInitPositiveLiteral" ):
                return visitor.visitInitPositiveLiteral(self)
            else:
                return visitor.visitChildren(self)



    def init_element(self):

        localctx = fstripsParser.Init_elementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_init_element)
        try:
            self.state = 641
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,50,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.InitPositiveLiteralContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 629
                self.flat_atom()
                pass

            elif la_ == 2:
                localctx = fstripsParser.InitNegativeLiteralContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 630
                self.match(fstripsParser.T__0)
                self.state = 631
                self.match(fstripsParser.K_NOT)
                self.state = 632
                self.flat_atom()
                self.state = 633
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.InitFunctionAssignmentContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 635
                self.match(fstripsParser.T__0)
                self.state = 636
                self.match(fstripsParser.T__35)
                self.state = 637
                self.flat_term()
                self.state = 638
                self.constant_name()
                self.state = 639
                self.match(fstripsParser.T__2)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Flat_termContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def function_name(self):
            return self.getTypedRuleContext(fstripsParser.Function_nameContext,0)


        def constant_name(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Constant_nameContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Constant_nameContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_flat_term

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFlat_term" ):
                listener.enterFlat_term(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFlat_term" ):
                listener.exitFlat_term(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFlat_term" ):
                return visitor.visitFlat_term(self)
            else:
                return visitor.visitChildren(self)




    def flat_term(self):

        localctx = fstripsParser.Flat_termContext(self, self._ctx, self.state)
        self.enterRule(localctx, 108, self.RULE_flat_term)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 643
            self.match(fstripsParser.T__0)
            self.state = 644
            self.function_name()
            self.state = 648
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.NAME or _la==fstripsParser.NUMBER:
                self.state = 645
                self.constant_name()
                self.state = 650
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 651
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Flat_atomContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def predicate(self):
            return self.getTypedRuleContext(fstripsParser.PredicateContext,0)


        def constant_name(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.Constant_nameContext)
            else:
                return self.getTypedRuleContext(fstripsParser.Constant_nameContext,i)


        def getRuleIndex(self):
            return fstripsParser.RULE_flat_atom

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFlat_atom" ):
                listener.enterFlat_atom(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFlat_atom" ):
                listener.exitFlat_atom(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFlat_atom" ):
                return visitor.visitFlat_atom(self)
            else:
                return visitor.visitChildren(self)




    def flat_atom(self):

        localctx = fstripsParser.Flat_atomContext(self, self._ctx, self.state)
        self.enterRule(localctx, 110, self.RULE_flat_atom)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 653
            self.match(fstripsParser.T__0)
            self.state = 654
            self.predicate()
            self.state = 658
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==fstripsParser.NAME or _la==fstripsParser.NUMBER:
                self.state = 655
                self.constant_name()
                self.state = 660
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 661
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Constant_nameContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_constant_name

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class Numeric_constantContext(Constant_nameContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Constant_nameContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(fstripsParser.NUMBER, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumeric_constant" ):
                listener.enterNumeric_constant(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumeric_constant" ):
                listener.exitNumeric_constant(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNumeric_constant" ):
                return visitor.visitNumeric_constant(self)
            else:
                return visitor.visitChildren(self)


    class Symbolic_constantContext(Constant_nameContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.Constant_nameContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSymbolic_constant" ):
                listener.enterSymbolic_constant(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSymbolic_constant" ):
                listener.exitSymbolic_constant(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSymbolic_constant" ):
                return visitor.visitSymbolic_constant(self)
            else:
                return visitor.visitChildren(self)



    def constant_name(self):

        localctx = fstripsParser.Constant_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 112, self.RULE_constant_name)
        try:
            self.state = 665
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [fstripsParser.NAME]:
                localctx = fstripsParser.Symbolic_constantContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 663
                self.match(fstripsParser.NAME)
                pass
            elif token in [fstripsParser.NUMBER]:
                localctx = fstripsParser.Numeric_constantContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 664
                self.match(fstripsParser.NUMBER)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class GoalContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_goal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGoal" ):
                listener.enterGoal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGoal" ):
                listener.exitGoal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitGoal" ):
                return visitor.visitGoal(self)
            else:
                return visitor.visitChildren(self)




    def goal(self):

        localctx = fstripsParser.GoalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 114, self.RULE_goal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 667
            self.match(fstripsParser.T__0)
            self.state = 668
            self.match(fstripsParser.T__45)
            self.state = 669
            self.goalDesc()
            self.state = 670
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ProbConstraintsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def prefConGD(self):
            return self.getTypedRuleContext(fstripsParser.PrefConGDContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_probConstraints

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProbConstraints" ):
                listener.enterProbConstraints(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProbConstraints" ):
                listener.exitProbConstraints(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProbConstraints" ):
                return visitor.visitProbConstraints(self)
            else:
                return visitor.visitChildren(self)




    def probConstraints(self):

        localctx = fstripsParser.ProbConstraintsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 116, self.RULE_probConstraints)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 672
            self.match(fstripsParser.T__0)
            self.state = 673
            self.match(fstripsParser.T__46)
            self.state = 674
            self.prefConGD()
            self.state = 675
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class PrefConGDContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_prefConGD

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class UniversallyQuantifiedConstraintContext(PrefConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PrefConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_FORALL(self):
            return self.getToken(fstripsParser.K_FORALL, 0)
        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)

        def prefConGD(self):
            return self.getTypedRuleContext(fstripsParser.PrefConGDContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterUniversallyQuantifiedConstraint" ):
                listener.enterUniversallyQuantifiedConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitUniversallyQuantifiedConstraint" ):
                listener.exitUniversallyQuantifiedConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitUniversallyQuantifiedConstraint" ):
                return visitor.visitUniversallyQuantifiedConstraint(self)
            else:
                return visitor.visitChildren(self)


    class PlainConstraintListContext(PrefConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PrefConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def conGD(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.ConGDContext)
            else:
                return self.getTypedRuleContext(fstripsParser.ConGDContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlainConstraintList" ):
                listener.enterPlainConstraintList(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlainConstraintList" ):
                listener.exitPlainConstraintList(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPlainConstraintList" ):
                return visitor.visitPlainConstraintList(self)
            else:
                return visitor.visitChildren(self)


    class ConjunctionOfConstraintsContext(PrefConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PrefConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_AND(self):
            return self.getToken(fstripsParser.K_AND, 0)
        def prefConGD(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.PrefConGDContext)
            else:
                return self.getTypedRuleContext(fstripsParser.PrefConGDContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConjunctionOfConstraints" ):
                listener.enterConjunctionOfConstraints(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConjunctionOfConstraints" ):
                listener.exitConjunctionOfConstraints(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConjunctionOfConstraints" ):
                return visitor.visitConjunctionOfConstraints(self)
            else:
                return visitor.visitChildren(self)


    class PreferenceConstraintContext(PrefConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.PrefConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def conGD(self):
            return self.getTypedRuleContext(fstripsParser.ConGDContext,0)

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPreferenceConstraint" ):
                listener.enterPreferenceConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPreferenceConstraint" ):
                listener.exitPreferenceConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPreferenceConstraint" ):
                return visitor.visitPreferenceConstraint(self)
            else:
                return visitor.visitChildren(self)



    def prefConGD(self):

        localctx = fstripsParser.PrefConGDContext(self, self._ctx, self.state)
        self.enterRule(localctx, 118, self.RULE_prefConGD)
        self._la = 0 # Token type
        try:
            self.state = 707
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,57,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.ConjunctionOfConstraintsContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 677
                self.match(fstripsParser.T__0)
                self.state = 678
                self.match(fstripsParser.K_AND)
                self.state = 682
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==fstripsParser.T__0:
                    self.state = 679
                    self.prefConGD()
                    self.state = 684
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 685
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.UniversallyQuantifiedConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 686
                self.match(fstripsParser.T__0)
                self.state = 687
                self.match(fstripsParser.K_FORALL)
                self.state = 688
                self.match(fstripsParser.T__0)
                self.state = 689
                self.possibly_typed_variable_list()
                self.state = 690
                self.match(fstripsParser.T__2)
                self.state = 691
                self.prefConGD()
                self.state = 692
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.PreferenceConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 694
                self.match(fstripsParser.T__0)
                self.state = 695
                self.match(fstripsParser.T__47)
                self.state = 697
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==fstripsParser.NAME:
                    self.state = 696
                    self.match(fstripsParser.NAME)


                self.state = 699
                self.conGD()
                self.state = 700
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 4:
                localctx = fstripsParser.PlainConstraintListContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 703 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 702
                        self.conGD()

                    else:
                        raise NoViableAltException(self)
                    self.state = 705 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,56,self._ctx)

                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class MetricSpecContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_metricSpec

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class ProblemMetricContext(MetricSpecContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.MetricSpecContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def optimization(self):
            return self.getTypedRuleContext(fstripsParser.OptimizationContext,0)

        def metricFExp(self):
            return self.getTypedRuleContext(fstripsParser.MetricFExpContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProblemMetric" ):
                listener.enterProblemMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProblemMetric" ):
                listener.exitProblemMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProblemMetric" ):
                return visitor.visitProblemMetric(self)
            else:
                return visitor.visitChildren(self)



    def metricSpec(self):

        localctx = fstripsParser.MetricSpecContext(self, self._ctx, self.state)
        self.enterRule(localctx, 120, self.RULE_metricSpec)
        try:
            localctx = fstripsParser.ProblemMetricContext(self, localctx)
            self.enterOuterAlt(localctx, 1)
            self.state = 709
            self.match(fstripsParser.T__0)
            self.state = 710
            self.match(fstripsParser.T__48)
            self.state = 711
            self.optimization()
            self.state = 712
            self.metricFExp()
            self.state = 713
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class OptimizationContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_optimization

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOptimization" ):
                listener.enterOptimization(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOptimization" ):
                listener.exitOptimization(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOptimization" ):
                return visitor.visitOptimization(self)
            else:
                return visitor.visitChildren(self)




    def optimization(self):

        localctx = fstripsParser.OptimizationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 122, self.RULE_optimization)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 715
            _la = self._input.LA(1)
            if not(_la==fstripsParser.T__49 or _la==fstripsParser.T__50):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class MetricFExpContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_metricFExp

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class FunctionalExprMetricContext(MetricFExpContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.MetricFExpContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunctionalExprMetric" ):
                listener.enterFunctionalExprMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunctionalExprMetric" ):
                listener.exitFunctionalExprMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunctionalExprMetric" ):
                return visitor.visitFunctionalExprMetric(self)
            else:
                return visitor.visitChildren(self)


    class CompositeMetricContext(MetricFExpContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.MetricFExpContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def terminalCost(self):
            return self.getTypedRuleContext(fstripsParser.TerminalCostContext,0)

        def stageCost(self):
            return self.getTypedRuleContext(fstripsParser.StageCostContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCompositeMetric" ):
                listener.enterCompositeMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCompositeMetric" ):
                listener.exitCompositeMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCompositeMetric" ):
                return visitor.visitCompositeMetric(self)
            else:
                return visitor.visitChildren(self)


    class IsViolatedMetricContext(MetricFExpContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.MetricFExpContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NAME(self):
            return self.getToken(fstripsParser.NAME, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIsViolatedMetric" ):
                listener.enterIsViolatedMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIsViolatedMetric" ):
                listener.exitIsViolatedMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitIsViolatedMetric" ):
                return visitor.visitIsViolatedMetric(self)
            else:
                return visitor.visitChildren(self)


    class TotalTimeMetricContext(MetricFExpContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.MetricFExpContext
            super().__init__(parser)
            self.copyFrom(ctx)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTotalTimeMetric" ):
                listener.enterTotalTimeMetric(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTotalTimeMetric" ):
                listener.exitTotalTimeMetric(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTotalTimeMetric" ):
                return visitor.visitTotalTimeMetric(self)
            else:
                return visitor.visitChildren(self)



    def metricFExp(self):

        localctx = fstripsParser.MetricFExpContext(self, self._ctx, self.state)
        self.enterRule(localctx, 124, self.RULE_metricFExp)
        try:
            self.state = 729
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,58,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.FunctionalExprMetricContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 717
                self.functionTerm()
                pass

            elif la_ == 2:
                localctx = fstripsParser.CompositeMetricContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 718
                self.terminalCost()
                self.state = 719
                self.stageCost()
                pass

            elif la_ == 3:
                localctx = fstripsParser.CompositeMetricContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 721
                self.stageCost()
                self.state = 722
                self.terminalCost()
                pass

            elif la_ == 4:
                localctx = fstripsParser.TotalTimeMetricContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 724
                self.match(fstripsParser.T__51)
                pass

            elif la_ == 5:
                localctx = fstripsParser.IsViolatedMetricContext(self, localctx)
                self.enterOuterAlt(localctx, 5)
                self.state = 725
                self.match(fstripsParser.T__0)
                self.state = 726
                self.match(fstripsParser.T__52)
                self.state = 727
                self.match(fstripsParser.NAME)
                self.state = 728
                self.match(fstripsParser.T__2)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class TerminalCostContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_terminalCost

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTerminalCost" ):
                listener.enterTerminalCost(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTerminalCost" ):
                listener.exitTerminalCost(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTerminalCost" ):
                return visitor.visitTerminalCost(self)
            else:
                return visitor.visitChildren(self)




    def terminalCost(self):

        localctx = fstripsParser.TerminalCostContext(self, self._ctx, self.state)
        self.enterRule(localctx, 126, self.RULE_terminalCost)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 731
            self.match(fstripsParser.T__0)
            self.state = 732
            self.match(fstripsParser.T__53)
            self.state = 733
            self.functionTerm()
            self.state = 734
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class StageCostContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def functionTerm(self):
            return self.getTypedRuleContext(fstripsParser.FunctionTermContext,0)


        def getRuleIndex(self):
            return fstripsParser.RULE_stageCost

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStageCost" ):
                listener.enterStageCost(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStageCost" ):
                listener.exitStageCost(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStageCost" ):
                return visitor.visitStageCost(self)
            else:
                return visitor.visitChildren(self)




    def stageCost(self):

        localctx = fstripsParser.StageCostContext(self, self._ctx, self.state)
        self.enterRule(localctx, 128, self.RULE_stageCost)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 736
            self.match(fstripsParser.T__0)
            self.state = 737
            self.match(fstripsParser.T__54)
            self.state = 738
            self.functionTerm()
            self.state = 739
            self.match(fstripsParser.T__2)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class ConGDContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return fstripsParser.RULE_conGD

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)



    class ForallConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_FORALL(self):
            return self.getToken(fstripsParser.K_FORALL, 0)
        def possibly_typed_variable_list(self):
            return self.getTypedRuleContext(fstripsParser.Possibly_typed_variable_listContext,0)

        def conGD(self):
            return self.getTypedRuleContext(fstripsParser.ConGDContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterForallConstraint" ):
                listener.enterForallConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitForallConstraint" ):
                listener.exitForallConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitForallConstraint" ):
                return visitor.visitForallConstraint(self)
            else:
                return visitor.visitChildren(self)


    class AlwaysConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAlwaysConstraint" ):
                listener.enterAlwaysConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAlwaysConstraint" ):
                listener.exitAlwaysConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAlwaysConstraint" ):
                return visitor.visitAlwaysConstraint(self)
            else:
                return visitor.visitChildren(self)


    class AtEndConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtEndConstraint" ):
                listener.enterAtEndConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtEndConstraint" ):
                listener.exitAtEndConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtEndConstraint" ):
                return visitor.visitAtEndConstraint(self)
            else:
                return visitor.visitChildren(self)


    class ConjunctiveConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def K_AND(self):
            return self.getToken(fstripsParser.K_AND, 0)
        def conGD(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.ConGDContext)
            else:
                return self.getTypedRuleContext(fstripsParser.ConGDContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConjunctiveConstraint" ):
                listener.enterConjunctiveConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConjunctiveConstraint" ):
                listener.exitConjunctiveConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConjunctiveConstraint" ):
                return visitor.visitConjunctiveConstraint(self)
            else:
                return visitor.visitChildren(self)


    class SometimeConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSometimeConstraint" ):
                listener.enterSometimeConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSometimeConstraint" ):
                listener.exitSometimeConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSometimeConstraint" ):
                return visitor.visitSometimeConstraint(self)
            else:
                return visitor.visitChildren(self)


    class WithinConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(fstripsParser.NUMBER, 0)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWithinConstraint" ):
                listener.enterWithinConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWithinConstraint" ):
                listener.exitWithinConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitWithinConstraint" ):
                return visitor.visitWithinConstraint(self)
            else:
                return visitor.visitChildren(self)


    class HoldAfterConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(fstripsParser.NUMBER, 0)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHoldAfterConstraint" ):
                listener.enterHoldAfterConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHoldAfterConstraint" ):
                listener.exitHoldAfterConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitHoldAfterConstraint" ):
                return visitor.visitHoldAfterConstraint(self)
            else:
                return visitor.visitChildren(self)


    class SometimeBeforeConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSometimeBeforeConstraint" ):
                listener.enterSometimeBeforeConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSometimeBeforeConstraint" ):
                listener.exitSometimeBeforeConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSometimeBeforeConstraint" ):
                return visitor.visitSometimeBeforeConstraint(self)
            else:
                return visitor.visitChildren(self)


    class SometimeAfterConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSometimeAfterConstraint" ):
                listener.enterSometimeAfterConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSometimeAfterConstraint" ):
                listener.exitSometimeAfterConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSometimeAfterConstraint" ):
                return visitor.visitSometimeAfterConstraint(self)
            else:
                return visitor.visitChildren(self)


    class HoldDuringConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self, i:int=None):
            if i is None:
                return self.getTokens(fstripsParser.NUMBER)
            else:
                return self.getToken(fstripsParser.NUMBER, i)
        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHoldDuringConstraint" ):
                listener.enterHoldDuringConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHoldDuringConstraint" ):
                listener.exitHoldDuringConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitHoldDuringConstraint" ):
                return visitor.visitHoldDuringConstraint(self)
            else:
                return visitor.visitChildren(self)


    class AlwaysWithinConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(fstripsParser.NUMBER, 0)
        def goalDesc(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(fstripsParser.GoalDescContext)
            else:
                return self.getTypedRuleContext(fstripsParser.GoalDescContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAlwaysWithinConstraint" ):
                listener.enterAlwaysWithinConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAlwaysWithinConstraint" ):
                listener.exitAlwaysWithinConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAlwaysWithinConstraint" ):
                return visitor.visitAlwaysWithinConstraint(self)
            else:
                return visitor.visitChildren(self)


    class AlternativeAlwaysConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAlternativeAlwaysConstraint" ):
                listener.enterAlternativeAlwaysConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAlternativeAlwaysConstraint" ):
                listener.exitAlternativeAlwaysConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAlternativeAlwaysConstraint" ):
                return visitor.visitAlternativeAlwaysConstraint(self)
            else:
                return visitor.visitChildren(self)


    class AtMostOnceConstraintContext(ConGDContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a fstripsParser.ConGDContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def goalDesc(self):
            return self.getTypedRuleContext(fstripsParser.GoalDescContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtMostOnceConstraint" ):
                listener.enterAtMostOnceConstraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtMostOnceConstraint" ):
                listener.exitAtMostOnceConstraint(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtMostOnceConstraint" ):
                return visitor.visitAtMostOnceConstraint(self)
            else:
                return visitor.visitChildren(self)



    def conGD(self):

        localctx = fstripsParser.ConGDContext(self, self._ctx, self.state)
        self.enterRule(localctx, 130, self.RULE_conGD)
        self._la = 0 # Token type
        try:
            self.state = 817
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,60,self._ctx)
            if la_ == 1:
                localctx = fstripsParser.ConjunctiveConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 741
                self.match(fstripsParser.T__0)
                self.state = 742
                self.match(fstripsParser.K_AND)
                self.state = 744 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 743
                    self.conGD()
                    self.state = 746 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (_la==fstripsParser.T__0):
                        break

                self.state = 748
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 2:
                localctx = fstripsParser.ForallConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 750
                self.match(fstripsParser.T__0)
                self.state = 751
                self.match(fstripsParser.K_FORALL)
                self.state = 752
                self.match(fstripsParser.T__0)
                self.state = 753
                self.possibly_typed_variable_list()
                self.state = 754
                self.match(fstripsParser.T__2)
                self.state = 755
                self.conGD()
                self.state = 756
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 3:
                localctx = fstripsParser.AtEndConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 758
                self.match(fstripsParser.T__0)
                self.state = 759
                self.match(fstripsParser.T__55)
                self.state = 760
                self.goalDesc()
                self.state = 761
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 4:
                localctx = fstripsParser.AlwaysConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 763
                self.match(fstripsParser.T__0)
                self.state = 764
                self.match(fstripsParser.T__56)
                self.state = 765
                self.goalDesc()
                self.state = 766
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 5:
                localctx = fstripsParser.SometimeConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 5)
                self.state = 768
                self.match(fstripsParser.T__0)
                self.state = 769
                self.match(fstripsParser.T__57)
                self.state = 770
                self.goalDesc()
                self.state = 771
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 6:
                localctx = fstripsParser.WithinConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 6)
                self.state = 773
                self.match(fstripsParser.T__0)
                self.state = 774
                self.match(fstripsParser.T__58)
                self.state = 775
                self.match(fstripsParser.NUMBER)
                self.state = 776
                self.goalDesc()
                self.state = 777
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 7:
                localctx = fstripsParser.AtMostOnceConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 7)
                self.state = 779
                self.match(fstripsParser.T__0)
                self.state = 780
                self.match(fstripsParser.T__59)
                self.state = 781
                self.goalDesc()
                self.state = 782
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 8:
                localctx = fstripsParser.SometimeAfterConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 8)
                self.state = 784
                self.match(fstripsParser.T__0)
                self.state = 785
                self.match(fstripsParser.T__60)
                self.state = 786
                self.goalDesc()
                self.state = 787
                self.goalDesc()
                self.state = 788
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 9:
                localctx = fstripsParser.SometimeBeforeConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 9)
                self.state = 790
                self.match(fstripsParser.T__0)
                self.state = 791
                self.match(fstripsParser.T__61)
                self.state = 792
                self.goalDesc()
                self.state = 793
                self.goalDesc()
                self.state = 794
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 10:
                localctx = fstripsParser.AlwaysWithinConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 10)
                self.state = 796
                self.match(fstripsParser.T__0)
                self.state = 797
                self.match(fstripsParser.T__62)
                self.state = 798
                self.match(fstripsParser.NUMBER)
                self.state = 799
                self.goalDesc()
                self.state = 800
                self.goalDesc()
                self.state = 801
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 11:
                localctx = fstripsParser.HoldDuringConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 11)
                self.state = 803
                self.match(fstripsParser.T__0)
                self.state = 804
                self.match(fstripsParser.T__63)
                self.state = 805
                self.match(fstripsParser.NUMBER)
                self.state = 806
                self.match(fstripsParser.NUMBER)
                self.state = 807
                self.goalDesc()
                self.state = 808
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 12:
                localctx = fstripsParser.HoldAfterConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 12)
                self.state = 810
                self.match(fstripsParser.T__0)
                self.state = 811
                self.match(fstripsParser.T__64)
                self.state = 812
                self.match(fstripsParser.NUMBER)
                self.state = 813
                self.goalDesc()
                self.state = 814
                self.match(fstripsParser.T__2)
                pass

            elif la_ == 13:
                localctx = fstripsParser.AlternativeAlwaysConstraintContext(self, localctx)
                self.enterOuterAlt(localctx, 13)
                self.state = 816
                self.goalDesc()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx





